<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\ControllerPlugin;

use XF\ControllerPlugin\AbstractPlugin;

class License extends AbstractPlugin
{
    public function getListData($type = 'all')
    {
        /** @var \XFA\RMMarketplace\Finder\License $licenseFinder */
        $licenseFinder = $this->finder('XFA\RMMarketplace:License');

        $filters = $this->getLicenseFilterInput();
        $this->applyLicenseFilters($licenseFinder, $filters, $type);

        $totalLicenses = $licenseFinder->total();

        $page       = $this->filterPage();
        $perPage    = $this->options()->xfrmResourcesPerPage;

        $licenseFinder->limitByPage($page, $perPage);
        $licenses = $licenseFinder->fetch();

        if (!empty($filters['user_id']))
        {
            $ownerFilter = $this->em()->find('XF:User', $filters['user_id']);
        }
        else
        {
            $ownerFilter = null;
        }

        if (!empty($filters['resource_id']))
        {
            $resourceFilter = $this->em()->find('XFRM:ResourceItem', $filters['resource_id']);
        }
        else
        {
            $resourceFilter = null;
        }

        return [
            'licenses'          => $licenses,
            'filters'           => $filters,
            'ownerFilter'       => $ownerFilter,
            'resourceFilter'    => $resourceFilter,
            'total'             => $totalLicenses,
            'page'              => $page,
            'perPage'           => $perPage
        ];
    }

    public function applyLicenseFilters(\XFA\RMMarketplace\Finder\License $licenseFinder, array $filters, $type)
    {
        $licenseFinder->with(['Resource','Resource.CurrentVersion','Resource.DigitalProduct'], true);

        $licenseFinder->where('validated', 1);

        if (!empty($filters['resource_id']))
        {
            $licenseFinder->where('resource_id', intval($filters['resource_id']));
        }

        if (!empty($filters['status']))
        {
            switch ($filters['status'])
            {
                case 'active':
                    $licenseFinder->where('active', '=', 1);
                    $licenseFinder->whereOr(['end_date', 0], ['end_date', '>', \XF::$time]);
                    break;

                case 'inactive':
                    $licenseFinder->where('active', '=', 0);
                    break;

                case 'expired':
                    $licenseFinder->where('active', '=', 1);
                    $licenseFinder->where('end_date', '<>', 0);
                    $licenseFinder->where('end_date', '<', \XF::$time);
                    break;
            }
        }

        if (!empty($filters['user_id']))
        {
            $licenseFinder->where('user_id', intval($filters['user_id']));
        }

        if (!empty($filters['direction']))
        {
            $licenseFinder->order('license_id', $filters['direction']);
        }

        if ($type == 'buyer')
        {
            $licenseFinder->where('user_id', \XF::visitor()->user_id);
        }
        else if ($type == 'seller')
        {
            $licenseFinder->where('Resource.user_id', \XF::visitor()->user_id);
        }
    }

    public function getLicenseFilterInput()
    {
        $filters = [];

        $input = $this->filter([
            'resource'      => 'str',
            'resource_id'   => 'uint',
            'status'        => 'str',
            'owner'         => 'str',
            'user_id'       => 'uint',
            'direction'     => 'str'
        ]);

        if ($input['resource_id'])
        {
            $filters['resource_id'] = $input['resource_id'];
        }
        else if ($input['resource'])
        {
            $resource = $this->em()->findOne('XFRM:ResourceItem', ['title' => $input['resource']]);
            if ($resource)
            {
                $filters['resource_id'] = $resource->resource_id;
            }
        }

        if ($input['status'] && ($input['status'] == 'active' || $input['status'] == 'inactive' || $input['status'] == 'expired'))
        {
            $filters['status'] = $input['status'];
        }

        if ($input['user_id'])
        {
            $filters['user_id'] = $input['user_id'];
        }
        else if ($input['owner'])
        {
            $user = $this->em()->findOne('XF:User', ['username' => $input['owner']]);
            if ($user)
            {
                $filters['user_id'] = $user->user_id;
            }
        }

        if ($input['direction'])
        {
            if (!in_array($input['direction'], ['asc', 'desc']))
            {
                $filters['direction'] = 'desc';
            }
            else
            {
                $filters['direction'] = $input['direction'];
            }
        }

        return $filters;
    }

    public function actionFilters($type, $baseUrl, $showUserFilter = false)
    {
        $filters = $this->getLicenseFilterInput();

        if ($this->filter('apply', 'bool'))
        {
            return $this->redirect($this->buildLink(
                $baseUrl,
                null,
                $filters
            ));
        }

        if (!empty($filters['user_id']))
        {
            $ownerFilter = $this->em()->find('XF:User', $filters['user_id']);
        }
        else
        {
            $ownerFilter = null;
        }

        /** @var \XFRM\Finder\ResourceItem $resourcesFinder */
        $resourcesFinder = $this->finder('XFRM:ResourceItem');

        $resourcesFinder->where('xfa_rmmp_type', 'digital');

        if ($type != 'all')
        {
            $resourcesFinder->where('user_id', \XF::visitor()->user_id);
        }

        $resources = $resourcesFinder->fetch()->filterViewable();

        $resourcesOptions = [];
        foreach($resources AS $resource)
        {
            $resourcesOptions[$resource->resource_id] = $resource->title;
        }

        $viewParams = [
            'baseUrl'           => $baseUrl,
            'resources'         => $resourcesOptions,
            'filters'           => $filters,
            'ownerFilter'       => $ownerFilter,
            'showUserFilter'    => $showUserFilter
        ];

        return $this->view('', 'xfa_rmmp_license_filters', $viewParams);
    }
}