<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\ControllerPlugin;

use XF\ControllerPlugin\AbstractPlugin;

class Purchase extends AbstractPlugin
{
    public function getFullListData($type = 'all', $onlyShowValidatedPurchases = true)
    {
        /** @var \XFA\RMMarketplace\Finder\Purchase $purchaseFinder */
        $purchaseFinder = $this->finder('XFA\RMMarketplace:Purchase');

        $filters = $this->getPurchaseFilterInput();
        $this->applyPurchaseFilters($purchaseFinder, $filters, $type, $onlyShowValidatedPurchases);

        $totalPurchases = $purchaseFinder->total();

        $purchases = $purchaseFinder->fetch();

        return $purchases;
    }

    public function getListData($type = 'all', $onlyShowValidatedPurchases = true)
    {
        /** @var \XFA\RMMarketplace\Finder\Purchase $purchaseFinder */
        $purchaseFinder = $this->finder('XFA\RMMarketplace:Purchase');
        $purchaseFinder->with(['Resource','Transaction','CouponUsage', 'CouponUsage.Coupon']);

        $filters = $this->getPurchaseFilterInput();
        $this->applyPurchaseFilters($purchaseFinder, $filters, $type, $onlyShowValidatedPurchases);

        $totalPurchases = $purchaseFinder->total();

        $page       = $this->filterPage();
        $perPage    = $this->options()->xfa_rmmp_purchasePerPage;

        $purchaseFinder->limitByPage($page, $perPage);
        $purchases = $purchaseFinder->fetch();

        if (!empty($filters['purchaser_id']))
        {
            $purchaseFilter = $this->em()->find('XF:User', $filters['purchaser_id']);
        }
        else
        {
            $purchaseFilter = null;
        }

        if (!empty($filters['resource_id']))
        {
            $resourceFilter = $this->em()->find('XFRM:ResourceItem', $filters['resource_id']);
        }
        else
        {
            $resourceFilter = null;
        }

        return [
            'purchases'         => $purchases,
            'filters'           => $filters,
            'purchaserFilter'   => $purchaseFilter,
            'resourceFilter'    => $resourceFilter,
            'total'             => $totalPurchases,
            'page'              => $page,
            'perPage'           => $perPage
        ];
    }

    public function applyPurchaseFilters(\XFA\RMMarketplace\Finder\Purchase $purchaseFinder, array $filters, $type, $onlyShowValidatedPurchases = true)
    {
        $purchaseFinder->with('Resource', true);
        $purchaseFinder->with('Resource.Seller');

        if (!empty($filters['resource_id']))
        {
            $purchaseFinder->where('resource_id', intval($filters['resource_id']));
        }

        if (!empty($filters['type']))
        {
            switch ($filters['type'])
            {
                case 'purchase':
                    $purchaseFinder->where('type', 'purchase');
                    break;

                case 'renewal':
                    $purchaseFinder->where('type', 'renewal');
                    break;
            }
        }

        if (!empty($filters['status']))
        {
            switch ($filters['status'])
            {
                case 'unpaid':
                case 'validated':
                case 'refunded':
                    $purchaseFinder->where('status', $filters['status']);
                    break;
            }
        }

        if (!empty($filters['shipment_status']))
        {
            switch ($filters['shipment_status'])
            {
                case 'pending':
                case 'in_preparation':
                case 'sent':
                case 'delivered':
                    $purchaseFinder->where('shipment_status', $filters['shipment_status']);
                    break;
            }
        }

        if (!empty($filters['purchaser_id']))
        {
            $purchaseFinder->where('user_id', intval($filters['purchaser_id']));
        }

        if ($onlyShowValidatedPurchases)
        {
            $purchaseFinder->where('status', 'validated');
        }

        if ($type == 'buyer')
        {
            $purchaseFinder->where('user_id', \XF::visitor()->user_id);
        }
        else if ($type == 'seller')
        {
            $purchaseFinder->where('Resource.user_id', \XF::visitor()->user_id);
        }

        if (!empty($filters['time_period']) && $filters['time_period'] != "all")
        {
            $purchaseFinder->where('purchase_date', '>', $filters['time_period_start']);
            $purchaseFinder->where('purchase_date', '<', $filters['time_period_end']);
        }

        if (!empty($filters['direction']))
        {
            $purchaseFinder->order('purchase_date', $filters['direction']);
        }
        else
        {
            $purchaseFinder->order('purchase_date', 'desc');
        }
    }

    public function getPurchaseFilterInput()
    {
        $filters = [];

        $input = $this->filter([
            'time_period'       => 'str',
            'time_period_start' => 'datetime',
            'time_period_end'   => 'datetime',
            'resource'          => 'str',
            'resource_id'       => 'uint',
            'type'              => 'str',
            'status'            => 'str',
            'shipment_status'   => 'str',
            'purchaser'         => 'str',
            'purchaser_id'      => 'uint',
            'direction'         => 'str'
        ]);

        $filters['time_period'] = $input['time_period'];

        if ($filters['time_period'] == "custom")
        {
            if ($input['time_period_start'] == 0
                || $input['time_period_end'] == 0
                || $input['time_period_start'] > $input['time_period_end'])
            {
                // Incorrect selected data, revert to all
                $filters['time_period'] = "all";
            }
            else
            {
                $filters['time_period_start']   = $input['time_period_start'];
                $filters['time_period_end']     = $input['time_period_end'];
            }
        }
        else if ($filters['time_period'] != "all")
        {
            // Get from/to based on type
            list($filters['time_period_start'], $filters['time_period_end']) = $this->getFromToBasedOnInput($input['time_period']);
        }

        if ($input['resource_id'])
        {
            $filters['resource_id'] = $input['resource_id'];
        }
        else if ($input['resource'])
        {
            $resource = $this->em()->findOne('XFRM:ResourceItem', ['title' => $input['resource']]);
            if ($resource)
            {
                $filters['resource_id'] = $resource->resource_id;
            }
        }

        if ($input['type'] && ($input['type'] == 'purchase' || $input['type'] == 'renewal'))
        {
            $filters['type'] = $input['type'];
        }

        if ($input['status'] && ($input['status'] == 'unpaid' || $input['status'] == 'validated' || $input['status'] == 'refunded'))
        {
            $filters['status'] = $input['status'];
        }

        if ($input['shipment_status'] && ($input['shipment_status'] == 'pending' || $input['shipment_status'] == 'in_preparation' || $input['shipment_status'] == 'sent' || $input['shipment_status'] == 'delivered'))
        {
            $filters['shipment_status'] = $input['shipment_status'];
        }

        if ($input['purchaser_id'])
        {
            $filters['purchaser_id'] = $input['purchaser_id'];
        }
        else if ($input['purchaser'])
        {
            $user = $this->em()->findOne('XF:User', ['username' => $input['purchaser']]);
            if ($user)
            {
                $filters['purchaser_id'] = $user->user_id;
            }
        }

        if ($input['direction'])
        {
            if (!in_array($input['direction'], ['asc', 'desc']))
            {
                $filters['direction'] = 'desc';
            }
            else
            {
                $filters['direction'] = $input['direction'];
            }
        }

        return $filters;
    }

    protected function getFromToBasedOnInput($type = 'current_month')
    {
        $timestamp  = \XF::$time;
        $dtz        = new \DateTimeZone(\XF::visitor()->timezone);

        /* Compute values */
        switch ($type)
        {
            case 'current_week':
                $from = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('monday this week')->format('Y-m-d 00:00:00 O'))->getTimestamp();
                $to   = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('sunday this week')->format('Y-m-d 23:59:59 O'))->getTimestamp();
                break;
            case 'current_quarter':
                $month = (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->format('n') ;

                if ($month < 4)
                {
                    $start  = 'first day of january';
                    $end    = 'last day of march';
                }
                elseif ($month > 3 && $month < 7)
                {
                    $start  = 'first day of april';
                    $end    = 'last day of june';
                }
                elseif ($month > 6 && $month < 10)
                {
                    $start  = 'first day of july';
                    $end    = 'last day of september';
                }
                elseif ($month > 9)
                {
                    $start  = 'first day of october';
                    $end    = 'last day of december';
                }

                $from = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify($start)->format('Y-m-d 00:00:00 O'))->getTimestamp();
                $to   = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify($end)->format('Y-m-d 23:59:59 O'))->getTimestamp();
                break;
            case 'current_year':
                $from = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('first day of january')->format('Y-m-d 00:00:00 O'))->getTimestamp();
                $to   = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('last day of december')->format('Y-m-d 23:59:59 O'))->getTimestamp();
                break;
            case 'previous_week':
                $from = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('monday last week')->format('Y-m-d 00:00:00 O'))->getTimestamp();
                $to   = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('sunday last week')->format('Y-m-d 23:59:59 O'))->getTimestamp();
                break;
            case 'previous_month':
                $from = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('first day of previous month')->format('Y-m-d 00:00:00 O'))->getTimestamp();
                $to   = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('last day of previous month')->format('Y-m-d 23:59:59 O'))->getTimestamp();
                break;
            case 'previous_quarter':
                $month = (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->format('n') ;

                if ($month < 4)
                {
                    $start  = 'first day of october last year';
                    $end    = 'last day of december last year';
                }
                elseif ($month > 3 && $month < 7)
                {
                    $start  = 'first day of january';
                    $end    = 'last day of march';
                }
                elseif ($month > 6 && $month < 10)
                {
                    $start  = 'first day of april';
                    $end    = 'last day of june';
                }
                elseif ($month > 9)
                {
                    $start  = 'first day of july';
                    $end    = 'last day of september';
                }

                $from = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify($start)->format('Y-m-d 00:00:00 O'))->getTimestamp();
                $to   = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify($end)->format('Y-m-d 23:59:59 O'))->getTimestamp();
                break;
            case 'previous_year':
                $from = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('first day of january last year')->format('Y-m-d 00:00:00 O'))->getTimestamp();
                $to   = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('last day of december last year')->format('Y-m-d 23:59:59 O'))->getTimestamp();
                break;
            case 'current_month':
            default:
                $from = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('first day of this month')->format('Y-m-d 00:00:00 O'))->getTimestamp();
                $to   = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('last day of this month')->format('Y-m-d 23:59:59 O'))->getTimestamp();
                break;
        }

        return array(
            $from,
            $to
        );
    }
    public function actionFilters($baseUrl, $inDashboard)
    {
        $filters = $this->getPurchaseFilterInput();

        if ($this->filter('apply', 'bool'))
        {
            return $this->redirect($this->buildLink(
                $baseUrl,
                null,
                $filters
            ));
        }

        if (!empty($filters['purchaser_id']))
        {
            $purchaserFilter = $this->em()->find('XF:User', $filters['purchaser_id']);
        }
        else
        {
            $purchaserFilter = null;
        }

        /** @var \XFRM\Finder\ResourceItem $resourcesFinder */
        $resourcesFinder = $this->finder('XFRM:ResourceItem');

        $resources = $resourcesFinder->where('xfa_rmmp_type', '<>', 'none')
                                     ->fetch()
                                     ->filterViewable();
        $resourcesOptions = [];
        foreach($resources AS $resource)
        {
            $resourcesOptions[$resource->resource_id] = $resource->title;
        }

        $viewParams = [
            'baseUrl'           => $baseUrl,
            'resources'         => $resourcesOptions,
            'filters'           => $filters,
            'purchaserFilter'   => $purchaserFilter,
            'inDashboard'       => $inDashboard
        ];

        return $this->view('', 'xfa_rmmp_purchase_filters', $viewParams);
    }
}