<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\ControllerPlugin;

use XF\ControllerPlugin\AbstractPlugin;

class Stats extends AbstractPlugin
{

    public function getListData($type = 'all')
    {
        $visitor = \XF::visitor();

        /* Get Filters */
        $filters = $this->getStatsFilterInput();

        /* Set the stats display types */
        $displayTypes['sales']  = ['nb_sales'];
        $displayTypes['income'] = ['gross', 'net'];

        $displayTypesPhrased['sales']   = ['nb_sales' => \XF::phrase('xfa_rmmp_nb_sales')->render()];
        $displayTypesPhrased['income']  = ['gross' => \XF::phrase('xfa_rmmp_gross')->render(), 'net' => \XF::phrase('xfa_rmmp_net')->render()];

        /* If we filter per resource, we need to also get the number of updates */
        if (isset($filters['resource_id']))
        {
            $displayTypes['sales'][]                    = 'updates';
            $displayTypesPhrased['sales']['updates']    = \XF::phrase('xfrm_updates')->render();
        }

        /* Get the stats based on filters */
        $data['sales']  = $this->getStatsBasedOnFilters($displayTypes['sales'], $filters, $type);
        $data['income'] = $this->getStatsBasedOnFilters($displayTypes['income'], $filters, $type);
        $globalStats    = $this->getGlobalStatsBasedOnFilters($filters, $type);

        return [
            'data'                  => $data,
            'datePresets'           => \XF::language()->getDatePresets(),
            'displayTypes'          => $displayTypes,
            'displayTypesPhrased'   => $displayTypesPhrased,
            'globalStats'           => $globalStats,
            'filters'               => $filters
        ];
    }

    public function getStatsBasedOnFilters($statsTypes, array $filters, $type)
    {
        /** @var \XF\Stats\Grouper\AbstractGrouper $grouper */
        $grouper = $this->app->create('stats.grouper', $filters['grouping']);

        /** @var \XF\Service\Stats\Grapher $grapher */
        $grapher = $this->service('XFA\RMMarketplace:Stats\Grapher', $filters['start'], $filters['end'], $statsTypes);

        $grapher->filterByCurrency($filters['currency']);

        if ($type == 'seller')
        {
            $grapher->filterByUser(\XF::visitor());
        }

        if (isset($filters['sale_type']))
        {
            $grapher->filterBySaleType($filters['sale_type']);
        }

        if (isset($filters['resource_id']))
        {
            $grapher->filterByResource($filters['resource_id']);
        }

        return $grapher->getGroupedData($grouper);
    }

    public function getGlobalStatsBasedOnFilters($filters, $type)
    {
        /** @var \XF\Service\Stats\Grapher $grapher */
        $grapher = $this->service('XFA\RMMarketplace:Stats\Grapher', $filters['start'], $filters['end']);

        $grapher->filterByCurrency($filters['currency']);

        if ($type == 'seller')
        {
            $grapher->filterByUser(\XF::visitor());
        }

        if (isset($filters['sale_type']))
        {
            $grapher->filterBySaleType($filters['sale_type']);
        }

        if (isset($filters['resource_id']))
        {
            $grapher->filterByResource($filters['resource_id']);
        }


        return $grapher->getSalesTotal();
    }

    public function getStatsFilterInput()
    {
        $filters = [];

        $input = $this->filter([
            'type'          => 'str',
            'grouping'      => 'str',
            'start'         => 'datetime',
            'end'           => 'datetime',
            'currency'      => 'str',
            'sale_type'     => 'str',
            'resource'      => 'str',
            'resource_id'   => 'uint',
        ]);

        /* If no data provided set default */
        if (!$input['type'])
        {
            if ($input['start'] == 0 || $input['end'] == 0 || $input['start'] > $input['end'] || !$input['grouping'])
            {
                $filters['type'] = \XF::options()->xfa_rmmp_default_stat_type;
                list($filters['grouping'], $filters['start'], $filters['end']) = $this->getStatsFromToBasedOnType(\XF::options()->xfa_rmmp_default_stat_type);
            }
            else
            {
                $filters['grouping']    = $input['grouping'];
                $filters['start']       = $input['start'];
                $filters['end']         = $input['end'];
                $filters['type']        = $input['type'];
            }
        }
        else
        {
            $filters['type'] = $input['type'];
            list($filters['grouping'], $filters['start'], $filters['end']) = $this->getStatsFromToBasedOnType($input['type']);
        }

        $filters['currency'] = $input['currency'] ? $input['currency'] : \XF::options()->xfa_rmmp_default_stat_currency;

        if ($input['sale_type'] && ($input['sale_type'] == 'purchase' || $input['sale_type'] == 'renewal'))
        {
            $filters['sale_type'] = $input['sale_type'];
        }

        if ($input['resource_id'])
        {
            $filters['resource_id'] = $input['resource_id'];
        }
        else if ($input['resource'])
        {
            $resource = $this->em()->findOne('XFRM:ResourceItem', ['title' => $input['resource']]);
            if ($resource)
            {
                $filters['resource_id'] = $resource->resource_id;
            }
        }

        return $filters;
    }

    protected function getStatsFromToBasedOnType($type = 'current_month')
    {
        $timestamp  = \XF::$time;
        $dtz        = new \DateTimeZone(\XF::visitor()->timezone);

        /* Compute values */
        switch ($type)
        {
            case 'current_week':
                $from = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('monday this week')->format('Y-m-d 00:00:00 O'))->getTimestamp();
                $to   = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('sunday this week')->format('Y-m-d 23:59:59 O'))->getTimestamp();
                $grouping = 'daily';
                break;
            case 'current_quarter':
                $month = (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->format('n') ;

                if ($month < 4)
                {
                    $start  = 'first day of january';
                    $end    = 'last day of march';
                }
                elseif ($month > 3 && $month < 7)
                {
                    $start  = 'first day of april';
                    $end    = 'last day of june';
                }
                elseif ($month > 6 && $month < 10)
                {
                    $start  = 'first day of july';
                    $end    = 'last day of september';
                }
                elseif ($month > 9)
                {
                    $start  = 'first day of october';
                    $end    = 'last day of december';
                }

                $from = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify($start)->format('Y-m-d 00:00:00 O'))->getTimestamp();
                $to   = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify($end)->format('Y-m-d 23:59:59 O'))->getTimestamp();
                $grouping = 'monthly';
                break;
            case 'current_year':
                $from = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('first day of january')->format('Y-m-d 00:00:00 O'))->getTimestamp();
                $to   = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('last day of december')->format('Y-m-d 23:59:59 O'))->getTimestamp();
                $grouping = 'monthly';
                break;
            case 'previous_week':
                $from = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('monday last week')->format('Y-m-d 00:00:00 O'))->getTimestamp();
                $to   = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('sunday last week')->format('Y-m-d 23:59:59 O'))->getTimestamp();
                $grouping = 'daily';
                break;
            case 'previous_month':
                $from = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('first day of previous month')->format('Y-m-d 00:00:00 O'))->getTimestamp();
                $to   = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('last day of previous month')->format('Y-m-d 23:59:59 O'))->getTimestamp();
                $grouping = 'daily';
                break;
            case 'previous_quarter':
                $month = (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->format('n') ;

                if ($month < 4)
                {
                    $start  = 'first day of october last year';
                    $end    = 'last day of december last year';
                }
                elseif ($month > 3 && $month < 7)
                {
                    $start  = 'first day of january';
                    $end    = 'last day of march';
                }
                elseif ($month > 6 && $month < 10)
                {
                    $start  = 'first day of april';
                    $end    = 'last day of june';
                }
                elseif ($month > 9)
                {
                    $start  = 'first day of july';
                    $end    = 'last day of september';
                }

                $from = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify($start)->format('Y-m-d 00:00:00 O'))->getTimestamp();
                $to   = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify($end)->format('Y-m-d 23:59:59 O'))->getTimestamp();
                $grouping = 'monthly';
                break;
            case 'previous_year':
                $from = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('first day of january last year')->format('Y-m-d 00:00:00 O'))->getTimestamp();
                $to   = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('last day of december last year')->format('Y-m-d 23:59:59 O'))->getTimestamp();
                $grouping = 'monthly';
                break;
            case 'current_month':
            default:
                $from = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('first day of this month')->format('Y-m-d 00:00:00 O'))->getTimestamp();
                $to   = \DateTime::createFromFormat('Y-m-d H:i:s O', (new \DateTime())->setTimezone($dtz)->setTimestamp($timestamp)->modify('last day of this month')->format('Y-m-d 23:59:59 O'))->getTimestamp();
                $grouping = 'daily';
                break;
        }

        return array(
            $grouping,
            $from,
            $to
        );
    }

    public function actionFilters($type)
    {
        $filters = $this->getStatsFilterInput();

        if ($this->filter('apply', 'bool'))
        {
            return $this->redirect($this->buildLink(
                'resources/market-place-dashboard/stats',
                null,
                $filters
            ));
        }

        /** @var \XFRM\Finder\ResourceItem $resourcesFinder */
        $resourcesFinder = $this->finder('XFRM:ResourceItem');

        if ($type != 'all')
        {
            $resourcesFinder->where('user_id', \XF::visitor()->user_id);
        }

        $resources = $resourcesFinder->fetch()->filterViewable();

        $resourcesOptions = [];
        foreach($resources AS $resource)
        {
            $resourcesOptions[$resource->resource_id] = $resource->title;
        }

        $viewParams = [
            'resources'         => $resourcesOptions,
            'filters'           => $filters
        ];

        return $this->view('', 'xfa_rmmp_stats_filters', $viewParams);
    }
}