<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2023
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

class Coupon extends Entity
{
    public function getPhrase()
    {
        $phrase  = \XF::phrase('xfa_rmmp_valid_coupon')->render();
        $phrase .= ' - ' . $this->getCouponTypePhrase();

        return $phrase;
    }

    public function getCouponTypePhrase()
    {
        switch ($this->coupon_type)
        {
            case 'free':
                $phrase = \XF::phrase('xfa_rmmp_free_product');
                break;
            case 'percentage':
                $phrase = \XF::phrase('xfa_rmmp_x_percent', ['amount' => $this->coupon_amount]);
                break;
            case 'discount':
                $phrase = \XF::phrase('xfa_rmmp_x_amount', ['amount' => $this->coupon_amount]);
                break;
        }

        return $phrase->render();
    }

    public function getTypePhrase()
    {
        switch ($this->type)
        {
            case 'all':
                $phraseName = 'xfa_rmmp_all_products';
                break;
            case 'all_own':
                $phraseName = 'xfa_rmmp_all_my_products';
                break;
            default:
                $phraseName = 'xfa_rmmp_specific_resources';
                break;
        }

        return \XF::phrase($phraseName)->render();
    }

    public function getApplyToPhrase()
    {
        switch ($this->apply_to)
        {
            case 'purchase':
                $phrase = \XF::phrase('xfa_rmmp_purchase')->render();
                break;
            case 'renewal':
                $phrase = \XF::phrase('xfa_rmmp_renewal')->render();
                break;
            default:
                $phrase = \XF::phrase('xfa_rmmp_purchase')->render() . ', ' . \XF::phrase('xfa_rmmp_renewal')->render();
                break;
        }

        return $phrase;
    }

    public function getRestrictedUsers()
    {
        if (!$this->user_ids)
        {
            return "";
        }

        $users = $this->finder('XF:User')->whereId($this->user_ids)->fetchColumns('username');

        $usernames = [];
        foreach($users AS $user)
        {
            $usernames[] = $user['username'];
        }

        return implode(', ', $usernames) . ', ';
    }

    function getUsergroupsRestrictionTooltip()
    {
        if (!$this->usergroups)
        {
            return "";
        }

        $usergroups = $this->finder('XF:UserGroup')->whereId($this->usergroups)->fetchColumns('title');

        $usergroupNames = [];
        foreach($usergroups AS $usergroup)
        {
            $usergroupNames[] = $usergroup['title'];
        }

        return implode(', ', $usergroupNames);
    }

    public function isUsableOnResourceByUser($type, $resource, \XF\Entity\User $user)
    {
        // Check usages
        if (count($this->UserUsages) >= $this->authorized_usages)
        {
            return false;
        }

        // Check date
        if (\XF::$time < $this->from_date
            || \XF::$time > $this->to_date)
        {
            return false;
        }

        // Check purchase type applicability
        if ($this->apply_to != 'both'
            && $type != $this->apply_to)
        {
            return false;
        }

        // Check type applicability
        if ($this->type == 'all_own')
        {
            if ($resource->user_id != $this->creator_id)
            {
                return false;
            }
        }
        else if ($this->type == 'specifics')
        {
            if (!in_array($resource->resource_id, $this->resource_ids))
            {
                return false;
            }
        }

        // Check user ids if restricted
        if (count($this->user_ids))
        {
            if (!in_array($user->user_id, $this->user_ids))
            {
                return false;
            }
        }

        // Check user groups if restricted
        if (count($this->usergroups))
        {
            if (!$user->isMemberOf($this->usergroups))
            {
                return false;
            }
        }

        return true;
    }

    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_xfa_rmmp_coupon';
        $structure->shortName = 'XFA\RMMarketplace:Coupon';
        $structure->primaryKey = 'coupon_id';
        $structure->columns = [
            'coupon_id'         => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'name'              => ['type' => self::STR, 'default' => '', 'maxLength' => 100, 'required' => true],
            'code'              => ['type' => self::STR, 'default' => '', 'maxLength' => 100, 'required' => true],
            'creator_id'        => ['type' => self::UINT, 'required' => true],
            'coupon_type'              => ['type' => self::STR, 'required' => true, 'default' => 'percentage',
                'allowedValues' => ['free', 'percentage', 'discount']
            ],
            'coupon_amount'     => ['type' => self::FLOAT, 'default' => 0, 'max' => 99999999, 'min' => 0],
            'apply_to'              => ['type' => self::STR, 'required' => true, 'default' => 'purchase',
                'allowedValues' => ['purchase', 'renewal', 'both']
            ],
            'type'              => ['type' => self::STR, 'required' => true, 'default' => 'all_own',
                'allowedValues' => ['all', 'all_own', 'specifics']
            ],
            'resource_ids'      => ['type' => self::LIST_COMMA, 'default' => []],
            'from_date'         => ['type' => self::UINT, 'default' => \XF::$time, 'required' => true],
            'to_date'           => ['type' => self::UINT, 'default' => (\XF::$time + 86400), 'required' => true],
            'usergroups'        => ['type' => self::LIST_COMMA, 'default' => []],
            'user_ids'          => ['type' => self::LIST_COMMA, 'default' => []],
            'authorized_usages' => ['type' => self::UINT, 'default' => 1],
            'total_usages'      => ['type' => self::UINT, 'default' => 0]
        ];
        $structure->getters = [];
        $structure->relations = [
            'UserUsages' => [
                'entity'        => 'XFA\RMMarketplace:CouponUsage',
                'type'          => self::TO_MANY,
                'conditions'    => [
                    ['coupon_id', '=', '$coupon_id'],
                    ['user_id', '=', \XF::visitor()->user_id]
                ]
            ]
        ];

        return $structure;
    }
}