<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;
use XFA\RMMarketplace\Entity\License;

class DigitalProduct extends Entity
{
    public function hasValidLicense()
    {
        if (!$this->ValidLicense)
        {
            return false;
        }

        /* Check non expired license if time limited */
        if ($this->duration != 0 && $this->ValidLicense->isExpired())
        {
            return false;
        }

        /* Check site url if url restricted */
        if ($this->require_url && $this->ValidLicense->hasNoUrlSet())
        {
            return false;
        }

        return true;
    }

    public function isPurchasable()
    {
        // Restricted products are always purchasable
        if ($this->has_restriction)
        {
            return true;
        }

        // If valid license, no need to purchase again
        if ($this->ValidLicense)
        {
            return false;
        }
        else
        {
            return true;
        }
    }

    public function getDurationString()
    {
        return $this->duration . ' ' . $this->duration_type;
    }

    public function getValidLicense()
    {
        if ($this->duration == 0)
        {
            return $this->ValidLicenseUnlimited;
        }
        else
        {
            return $this->ValidLicenseTimeLimited;
        }
    }

    public function applyDiscount($discount)
    {
        // Backup current price
        $this->base_renewal_price = $this->renewal_price;

        // Apply discount
        if ($discount->is_fixed)
        {
            $this->renewal_price = $this->renewal_price - $discount->discount_amount;
        }
        else
        {
            $this->renewal_price = $this->renewal_price * (1.0 - ($discount->discount_amount / 100.0));
        }
    }

    public function removeDiscount()
    {
        if ($this->renewal_price != $this->base_renewal_price
                && $this->base_renewal_price != 0)
        {
            $this->renewal_price = $this->base_renewal_price;
        }
    }

    public function getComputedRenewalPrice($coupon)
    {
        $price = $this->renewal_price;

        if ($coupon)
        {
            switch($coupon->coupon_type)
            {
                case 'discount':
                    $price = $price - $coupon->coupon_amount;
                    break;
                case 'percentage':
                    $price = $price - ($price * ($coupon->coupon_amount / 100));
                    break;
            }
        }

        return $price;
    }

    public function getRenewableLicenses($userId)
    {
        // Get licenses
        $licensesFinder = $this->finder('XFA\RMMarketplace:License');
        $licensesFinder->where('resource_id', $this->resource_id);
        $licensesFinder->where('user_id', $userId);
        $licensesFinder->where('validated', 1);
        $licensesFinder->where('active', 1);

        $licenses = $licensesFinder->fetch();

        // Filter out the non renewable
        $licenses = $licenses->filter(function(License $license)
        {
            if ($license->canBeRenewed())
            {
                return $license;
            }

            return null;
        });

        if ($licenses->count() > 1)
        {
            return $licenses;
        }
        else
        {
            return $licenses->first();
        }
    }

    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_xfa_rmmp_digital_product';
        $structure->shortName = 'XFA\RMMarketplace:DigitalProduct';
        $structure->primaryKey = 'digitalp_id';
        $structure->columns = [
            'digitalp_id'           => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'resource_id'           => ['type' => self::UINT, 'required' => true],
            'duration'              => ['type' => self::UINT, 'required' => true],
            'duration_type'         => ['type' => self::STR, 'default' => 'day',
                'allowedValues' => ['day', 'week', 'month', 'year']
            ],
            'base_renewal_price'    => ['type' => self::FLOAT, 'required' => true, 'default' => 0],
            'renewal_price'         => ['type' => self::FLOAT, 'required' => true, 'default' => 0],
            'delay_renewal'         => ['type' => self::UINT, 'required' => true, 'default' => 0],
            'allow_expired_renewal' => ['type' => self::BOOL, 'required' => true, 'default' => true],
            'has_restriction'       => ['type' => self::BOOL, 'required' => true],
            'require_url'           => ['type' => self::BOOL, 'required' => true],
            'lic_key_format'        => ['type' => self::STR]
        ];
        $structure->getters = ['ValidLicense' => true];
        $structure->relations = [
            'ValidLicenseTimeLimited' => [
                'entity'        => 'XFA\RMMarketplace:License',
                'type'          => Entity::TO_ONE,
                'conditions'    => [
                    ['resource_id', '=', '$resource_id'],
                    ['user_id', '=', \XF::visitor()->user_id],
                    ['validated', '=', 1],
                    ['active', '=', 1],
                    ['end_date', '>', \XF::$time]
                ]
            ],
            'ValidLicenseUnlimited' => [
                'entity'        => 'XFA\RMMarketplace:License',
                'type'          => Entity::TO_ONE,
                'conditions'    => [
                    ['resource_id', '=', '$resource_id'],
                    ['user_id', '=', \XF::visitor()->user_id],
                    ['validated', '=', 1],
                    ['active', '=', 1]
                ]
            ],
            'ExpiredLicense' => [
                'entity'        => 'XFA\RMMarketplace:License',
                'type'          => Entity::TO_ONE,
                'conditions'    => [
                    ['resource_id', '=', '$resource_id'],
                    ['user_id', '=', \XF::visitor()->user_id],
                    ['validated', '=', 1],
                    ['active', '=', 1],
                    ['end_date', '<', \XF::$time]
                ]
            ],
        ];
        $structure->defaultWith = ['ValidLicenseUnlimited', 'ValidLicenseTimeLimited'];

        return $structure;
    }
}