<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Pub\Controller;

use XF\Mvc\ParameterBag;
use XF\Mvc\FormAction;
use XF\Pub\Controller\AbstractController;
use OneSheet\Writer;

class DashboardDiscount extends AbstractController
{
    protected function preDispatchController($action, ParameterBag $params)
    {
        $this->assertRegistrationRequired();

        /** @var \XFRM\XF\Entity\User $visitor */
        $visitor = \XF::visitor();

        if (!$visitor->canViewResources($error)
            || !($visitor->isASeller()
                    || $visitor->canViewAllSales()
                    || $visitor->canManageAllLicenses()))
        {
            throw $this->exception($this->noPermission($error));
        }
    }

    public function actionIndex()
    {
        $visitor = \XF::visitor();
        $options = \XF::options();

        if (!$visitor->isASeller())
        {
            throw $this->exception($this->noPermission());
        }

        /** @var \XFA\RMMarketplace\ControllerPlugin\Discount $discountPlugin */
        $discountPlugin = $this->plugin('XFA\RMMarketplace:Discount');

        if ($visitor->canViewAllSales() && $this->request()->getCookie('xfa_rmmp_dashboard_conf') == 'all')
        {
            $viewParams = $discountPlugin->getListData('all', false);
        }
        else
        {
            $viewParams = $discountPlugin->getListData('seller', false);
        }

        $this->assertValidPage($viewParams['page'], $viewParams['perPage'], $viewParams['total'], 'resources/market-place-dashboard/discount');
        $this->assertCanonicalUrl($this->buildLink('resources/market-place-dashboard/discount', null, ['page' => $viewParams['page']]));

        return $this->view('', 'xfa_rmmp_dashboard_discounts', $viewParams);
    }

    public function actionFilters()
    {
        /** @var \XFA\RMMarketplace\ControllerPlugin\Discount $discountPlugin */
        $discountPlugin = $this->plugin('XFA\RMMarketplace:Discount');

        if (\XF::visitor()->canViewAllSales() && $this->request()->getCookie('xfa_rmmp_dashboard_conf') == 'all')
        {
            return $discountPlugin->actionFilters('all');
        }
        else
        {
            return $discountPlugin->actionFilters('seller');
        }
    }

    public function actionAdd()
    {
        $visitor = \XF::visitor();

        if (!$visitor->isASeller())
        {
            throw $this->exception($this->noPermission());
        }

        if ($this->isPost())
        {
            $this->setupAdd()->run();

            return $this->redirect($this->buildLink('resources/market-place-dashboard/discount'));
        }
        else
        {
            if (\XF::visitor()->canViewAllSales() && $this->request()->getCookie('xfa_rmmp_dashboard_conf') == 'all')
            {
                $viewParams = ['hasAllRights' => true, 'discount' => $this->em()->create('XFA\RMMarketplace:Discount')];
            } else
            {
                $viewParams = ['hasAllRights' => false, 'discount' => $this->em()->create('XFA\RMMarketplace:Discount')];
            }

            return $this->view('', 'xfa_rmmp_dashboard_discount_add', $viewParams);
        }
    }

    public function actionEdit(ParameterBag $params)
    {
        $visitor = \XF::visitor();

        if (!$visitor->isASeller())
        {
            throw $this->exception($this->noPermission());
        }

        $discount = $this->finder('XFA\RMMarketplace:Discount')->whereId($params->discount_id)->fetchOne();
        if (!$discount)
        {
            throw $this->exception($this->noPermission());
        }

        if ($this->isPost())
        {
            $this->setupEdit($discount)->run();

            return $this->redirect($this->buildLink('resources/market-place-dashboard/discount'));
        }
        else
        {
            // Can't edit past end date discount
            if ($discount->to_date < \XF::$time)
            {
                throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_cant_edit_past_discount')));
            }

            $viewParams = [
                'discount' => $discount
            ];

            if (\XF::visitor()->canViewAllSales() && $this->request()->getCookie('xfa_rmmp_dashboard_conf') == 'all')
            {
                $viewParams['hasAllRights'] = true;
            } else
            {
                $viewParams['hasAllRights'] = false;
            }

            return $this->view('', 'xfa_rmmp_dashboard_discount_edit', $viewParams);
        }
    }

    public function actionDelete(ParameterBag $params)
    {
        $visitor = \XF::visitor();

        if (!$visitor->isASeller())
        {
            throw $this->exception($this->noPermission());
        }

        $discount = $this->finder('XFA\RMMarketplace:Discount')->whereId($params->discount_id)->fetchOne();
        if (!$discount)
        {
            throw $this->exception($this->noPermission());
        }

        if ($this->isPost())
        {
            $discount->delete();

            return $this->redirect($this->buildLink('resources/market-place-dashboard/discount'));
        }
        else
        {
            $viewParams = [
                'discount' => $discount
            ];

            return $this->view('', 'xfa_rmmp_dashboard_discount_delete', $viewParams);
        }
    }

    public function actionGetResources()
    {
        $q = ltrim($this->filter('q', 'str', ['no-trim']));

        if ($q !== '' && utf8_strlen($q) >= 2)
        {
            /** @var \XFRM\Finder\ResourceItem $resourceFinder */
            $resourceFinder = $this->finder('XFRM:ResourceItem');
            $resourceFinder->where('title', 'like', $resourceFinder->escapeLike($q, '?%'));
            $resourceFinder->where('xfa_rmmp_type', '<>', 'none');
            $resourceFinder->where('xfa_rmmp_discount_id', '=', '0');

            if (!\XF::visitor()->canViewAllSales() || $this->request()->getCookie('xfa_rmmp_dashboard_conf') != 'all')
            {
                $resourceFinder->where('user_id', '=', \XF::visitor()->user_id);
            }

            $resources = $resourceFinder->fetch();
        }
        else
        {
            $resources    = [];
            $q          = '';
        }

        $viewParams = [
            'q'         => $q,
            'resources' => $resources
        ];

        return $this->view('XFA\RMMarketplace:GetDiscountResources', '', $viewParams);
    }

    public function actionResources(ParameterBag $params)
    {
        $visitor = \XF::visitor();

        if (!$visitor->isASeller())
        {
            throw $this->exception($this->noPermission());
        }

        $discount = $this->finder('XFA\RMMarketplace:Discount')->whereId($params->discount_id)->fetchOne();
        if (!$discount || $discount->type != 'specifics')
        {
            throw $this->exception($this->noPermission());
        }

        $resources = $this->finder('XFRM:ResourceItem')->where('resource_id', '=', $discount->resource_ids)->fetch();

        $viewParams = [
            'resources' => $resources
        ];

        return $this->view('', 'xfa_rmmp_dashboard_discount_resources', $viewParams);
    }

    public function setupAdd()
    {
        $discount = $this->em()->create('XFA\RMMarketplace:Discount');

        $form = $this->formAction();

        $input = $this->filter([
            'name'              => 'str',
            'type'              => 'str',
            'discount_amount'   => 'float',
            'from_date'         => 'datetime',
            'to_date'           => 'datetime',
            'applyto_renewal'   => 'bool'
        ]);

        // Fixed ?
        if ($this->filter('discount_type', 'str') == 'amount')
        {
            $input['is_fixed'] = 1;
        }
        else
        {
            $input['is_fixed'] = 0;
        }

        // Check end date is above start date
        if ($input['to_date'] < $input['from_date'])
        {
            throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_end_date_must_be_after_start_date')));
        }

        // Check end date is not in the past
        if ($input['to_date'] < \XF::$time)
        {
            throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_end_date_cant_be_in_past')));
        }

        // If all, check permissions are ok
        if ($input['type'] == 'all' && (!\XF::visitor()->canViewAllSales()
                || $this->request()->getCookie('xfa_rmmp_dashboard_conf') != 'all'))
        {
            throw $this->exception($this->noPermission());
        }

        // Validate discount
        switch($input['type'])
        {
            case 'all':
            case 'all_own':
                // Check there are resources without discount applied
                $resourceFinder = $this->finder('XFRM:ResourceItem')
                    ->where('xfa_rmmp_type', '<>', 'none')
                    ->where('xfa_rmmp_discount_id', '=', 0);

                if ($input['type'] == 'all_own')
                {
                    $resourceFinder->where('user_id', '=', \XF::visitor()->user_id);
                }

                if ($resourceFinder->total() == 0)
                {
                    throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_no_valid_resource_found_for_discount')));
                }

                break;
            case 'specifics':
                $resourceNames = $this->filter('resources', 'str');
                $resourceNames = preg_split('#\s*,\s*#', $resourceNames, -1, PREG_SPLIT_NO_EMPTY);

                /** @var \XFRM\Repository\ResourceItem $resourceRepo */
                $resourceRepo = $this->repository('XFRM:ResourceItem');
                $resources    = $resourceRepo->getNonRMDiscountedResourcesByName($resourceNames, $notFound)->toArray();

                if ($notFound)
                {
                    throw $this->exception(
                        $this->noPermission(
                            \XF::phrase(
                                'xfa_rmmp_the_following_resources_x_could_not_be_found_or_are_already_discounted',
                                ['names' => implode(', ', $notFound)]
                            )
                        )
                    );
                }

                $input['resource_ids'] = array_keys($resources);

                break;
        }

        $input['creator_id'] = \XF::visitor()->user_id;

        $form->basicEntitySave($discount, $input);

        return $form;
    }

    public function setupEdit($discount)
    {
        $form = $this->formAction();

        $input = $this->filter([
            'name'              => 'str',
            'from_date'         => 'datetime',
            'to_date'           => 'datetime'
        ]);

        // Remove start date if current was in the past
        if ($discount->from_date < \XF::$time)
        {
            unset($input['from_date']);
        }
        else
        {
            // Check end date is above start date
            if ($input['to_date'] < $input['from_date'])
            {
                throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_end_date_must_be_after_start_date')));
            }
        }

        // Check end date is in the past
        if ($input['to_date'] < \XF::$time)
        {
            throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_end_date_cant_be_in_past')));
        }

        $form->basicEntitySave($discount, $input);

        return $form;
    }
}