<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Pub\Controller;

use XF\Mvc\ParameterBag;
use XF\Pub\Controller\AbstractController;
use XFA\RMMarketplace\Util\DomainValidator;

class License extends AbstractController
{
    protected function preDispatchController($action, ParameterBag $params)
    {
        $this->assertRegistrationRequired();

        $visitor = \XF::visitor();

        if (!$visitor->canViewResources())
        {
            throw $this->exception($this->noPermission());
        }
    }

    public function actionEdit(ParameterBag $params)
    {
        $license = $this->assertEditableLicense($params->license_id);

        if ($this->isPost())
        {
            /* Get params */
            $active     = $this->filter('active', 'bool');
            $siteUrl    = $this->filter('siteurl', 'str');
            $endDate    = $this->filter('end_date', 'datetime');

            /* Filter site url and remove http & https and trailing slash to ensure same starting point*/

            $siteUrl = str_replace('http://',  '', $siteUrl);
            $siteUrl = str_replace('https://', '', $siteUrl);

            $license = $this->setupLicenseEdit($license, $active, $siteUrl, $endDate)->save();

            return $this->redirect($this->buildLink('resources/market-place-dashboard/manage-licenses'));
        }
        else
        {
            $viewParams = [
                'license' => $license
            ];
            return $this->view('', 'xfa_rmmp_license_edit', $viewParams);
        }
    }

    public function actionSetUrl(ParameterBag $params)
    {
        $license = $this->assertOwnedLicense($params->license_id);

        if ($this->isPost())
        {
            /* Filter site url and remove http & https and trailing slash to ensure same starting point*/
            $siteUrl = $this->filter('siteurl', 'str');
            $siteUrl = str_replace('http://',  '', $siteUrl);
            $siteUrl = str_replace('https://', '', $siteUrl);

            /* Error if empty */
            if (empty($siteUrl))
            {
                return $this->error(\XF::phrase('xfa_rmmp_nourl_specified'));
            }

            /* Check domain validity */
            if (!DomainValidator::checkValidity($siteUrl, \XF::options()->xfa_rmmp_licence_verif_method, \XF::options()->xfa_rmmp_licence_blacklist))
            {
                return $this->error(\XF::phrase('xfa_rmmp_invalid_domain_provided'));
            }

            /* Update license */
            $license->siteurl = $siteUrl;
            $license->save();

            return $this->redirect($this->buildLink('resources/market-place-user/licenses'));
        }
        else
        {
            $viewParams = [
                'license' => $license
            ];

            return $this->view('', 'xfa_rmmp_license_url_set_form', $viewParams);
        }
    }

    public function actionTransfer(ParameterBag $params)
    {
        $license = $this->assertEditableLicense($params->license_id);

        if ($this->isPost())
        {
            // Check provided user exists
            $user = $this->em()->findOne('XF:User', ['username' => $this->filter('username', 'str')]);
            if (!$user)
            {
                throw $this->exception($this->notFound(\XF::phrase('requested_user_not_found')));
            }

            // Update license info
            $license->user_id   = $user->user_id;
            $license->username  = $user->username;
            $license->save();

            return $this->redirect($this->buildLink('resources/market-place-dashboard/manage-licenses'));
        }
        else
        {
            $viewParams = [
                'license' => $license
            ];

            return $this->view('', 'xfa_rmmp_license_transfer', $viewParams);
        }
    }

    public function actionRenewXf(ParameterBag $params)
    {
        $license 	= $this->assertRenewableLicense($params->license_id, ['Resource.Category']);
        $resource   = $license->Resource;

        // Check purchasability
        $purchasable = $this->em()->find('XF:Purchasable', 'xfa_rmmp_renew', 'AddOn');
        if (!$purchasable || !$purchasable->isActive())
        {
            return $this->message(\XF::phrase('xfa_rmmp_no_purchase_cant_be_done_right_now'));
        }

        // Check we have xF payment method activated
        if (\XF::options()->xfa_rmmp_mode != 'xf')
        {
            throw $this->exception($this->noPermission());
        }

        // Get payment profiles depending on configuration
        $profiles = null;
        switch ($resource->Category->xfa_rmmp_conf)
        {
            // User
            case 1:
                $profiles = $this->finder('XF:PaymentProfile')
                    ->where('provider_id', \XF::options()->xfa_rmmp_paymentProviderIds)
                    ->where('xfa_rmmp_user_id', $resource->user_id)
                    ->where('active', 1)
                    ->fetch();
                break;
            // Category
            case 2:
                $profiles = $this->finder('XF:PaymentProfile')
                    ->where('payment_profile_id', explode(',', $resource->Category->xfa_rmmp_payment_data))
                    ->where('active', 1)
                    ->fetch();
                break;
            // Forum
            case 3:
                $profiles = $this->finder('XF:PaymentProfile')
                    ->where('payment_profile_id', \XF::options()->xfa_rmmp_paymentProfileIds)
                    ->where('active', 1)
                    ->fetch();
                break;
        }

        if (!count($profiles))
        {
            throw $this->exception($this->noPermission());
        }

        $viewParams = [
            'license'	=> $license,
            'resource'  => $resource,
            'profiles'  => $profiles
        ];
        return $this->view('', 'xfa_rmmp_renew_xf', $viewParams);
    }

    public function actionRenew(ParameterBag $params)
    {
        $license = $this->assertRenewableLicense($params->license_id, ['Resource.Category']);

        // Check we have custom payment method activated
        if (\XF::options()->xfa_rmmp_mode != 'custom')
        {
            throw $this->exception($this->noPermission());
        }

        // Show form
        $viewParams = [
            'license'   => $license,
            'resource'  => $license->Resource
        ];

        return $this->view('', 'xfa_rmmp_renew_custom', $viewParams);
    }

    public function actionCustomRenew(ParameterBag $params)
    {
        $license    = $this->assertRenewableLicense($params->license_id, ['Resource.Category']);
        $resource   = $license->Resource;

        // Check we have custom payment method activated
        if (\XF::options()->xfa_rmmp_mode != 'custom')
        {
            throw $this->exception($this->noPermission());
        }

        // Get coupon if any
        $couponCode = $this->filter('coupon_code', 'str');
        $coupon     = \XF::finder('XFA\RMMarketplace:Coupon')
            ->where('code', $couponCode)
            ->fetchOne();

        // Check coupon is usable on resource
        $usableCoupon = $coupon ? $coupon->isUsableOnResourceByUser('renewal', $resource, \XF::visitor()) : false;
        if ($couponCode && !$usableCoupon)
        {
            $error = \XF::phrase('xfa_rmmp_invalid_coupon');
            return false;
        }

        // Create the transaction
        $transaction = $this->setupTransactionCreate($resource)->save();

        // Create the purchase
        $purchase = $this->setupPurchaseRenewCreate($resource, $transaction, $license, $coupon)->save();

        // Check if percentage on sales for category or globally
        if ($resource->bypassPercentageOnSale())
        {
            $withPercentageOnSale       = 0;
            $percentageOnSale           = 0;
            $mailForPercentageOnSale    = '';
        }
        else
        {
            if ($resource->Category->xfa_rmmp_sale_percentage > 0.00)
            {
                $withPercentageOnSale       = 1;
                $percentageOnSale           = $resource->Category->xfa_rmmp_sale_percentage;
                $mailForPercentageOnSale    = $resource->Category->xfa_rmmp_sale_percentage_paypal;
            }
            else if (\XF::options()->xfa_rmmp_sale_percentage > 0.00)
            {
                $withPercentageOnSale       = 1;
                $percentageOnSale           = \XF::options()->xfa_rmmp_sale_percentage;
                $mailForPercentageOnSale    = \XF::options()->xfa_rmmp_paypal;
            }
            else
            {
                $withPercentageOnSale       = 0;
                $percentageOnSale           = 0;
                $mailForPercentageOnSale    = '';
            }
        }

        // Compute amounts based on that
        if ($withPercentageOnSale)
        {
            $adminAmount    = round(($purchase->price * $percentageOnSale / 100), 2);
            $sellerAmount   = $purchase->price - $adminAmount;
        }
        else
        {
            $adminAmount    = 0;
            $sellerAmount   = $purchase->price;
        }

        // Configure set express checkout call
        $setExpressCheckout = $this->service('XFA\Core:PayPal\SetExpressCheckout', \XF::options()->xfa_rmmp_paypal_testing);

        $setExpressCheckout->setUrls(
            $this->buildLink('canonical:resources/market-place-license/renew-done', $license),
            $this->buildLink('canonical:resources', $resource)
        );

        $setExpressCheckout->setBuyerEmail(\XF::visitor()->email);

        $setExpressCheckout->setItem(
            $transaction->transaction_id,
            $resource->title,
            $sellerAmount,
            $resource->xfa_rmmp_currency,
            $resource->xfa_rmmp_paypal,
            $this->buildLink('canonical:resources', $resource),
            \XF::phrase('xfa_rmmp_renewal') . ': ' . $resource->title
        );

        if ($withPercentageOnSale)
        {
            $setExpressCheckout->setFees($adminAmount, $resource->xfa_rmmp_currency, $mailForPercentageOnSale);
        }

        // Perform the call and check for errors
        $setExpressCheckout->run();

        if ($setExpressCheckout->hasErrors())
        {
            return $this->error($setExpressCheckout->getErrors());
        }

        // Update transaction and redirect to paypal
        $transaction->token = $setExpressCheckout->getToken();
        $transaction->save();

        $viewParams = [
            'resource'  => $resource,
            'redirect'  => $setExpressCheckout->getRedirectUrl()
        ];
        return $this->view('', 'xfa_core_paypal_express_checkout_redirect', $viewParams);
    }

    public function actionRenewDone(ParameterBag $params)
    {
        $license    = $this->assertRenewableLicense($params->license_id, ['Resource.Category']);
        $resource   = $license->Resource;

        // Check we have custom payment method activated
        if (\XF::options()->xfa_rmmp_mode != 'custom')
        {
            throw $this->exception($this->noPermission());
        }

        // Check provided token correspond to an existing transaction
        $token = $this->filter('token', 'str');

        $transaction = \XF::finder('XFA\RMMarketplace:Transaction')->where('token', $token)->fetchOne();
        if (!$transaction)
        {
            return $this->error(\XF::phrase('xfa_rmmp_transaction_doesnt_exist'));
        }

        // Check transaction and resource are matching
        if (!$transaction->Purchase || $transaction->Purchase->resource_id != $resource->resource_id)
        {
            return $this->error(\XF::phrase('xfa_rmmp_purchase_and_resource_do_not_match'));
        }

        // Check if percentage on sales for category or globally
        if ($resource->bypassPercentageOnSale())
        {
            $withPercentageOnSale       = 0;
            $percentageOnSale           = 0;
            $mailForPercentageOnSale    = '';
        }
        else
        {
            if ($resource->Category->xfa_rmmp_sale_percentage > 0.00)
            {
                $withPercentageOnSale       = 1;
                $percentageOnSale           = $resource->Category->xfa_rmmp_sale_percentage;
                $mailForPercentageOnSale    = $resource->Category->xfa_rmmp_sale_percentage_paypal;
            }
            else if (\XF::options()->xfa_rmmp_sale_percentage > 0.00)
            {
                $withPercentageOnSale       = 1;
                $percentageOnSale           = \XF::options()->xfa_rmmp_sale_percentage;
                $mailForPercentageOnSale    = \XF::options()->xfa_rmmp_paypal;
            }
            else
            {
                $withPercentageOnSale       = 0;
                $percentageOnSale           = 0;
                $mailForPercentageOnSale    = '';
            }
        }

        // Compute amounts based on that
        if ($withPercentageOnSale)
        {
            $adminAmount    = round(($transaction->Purchase->price * $percentageOnSale / 100), 2);
            $sellerAmount   = $transaction->Purchase->price - $adminAmount;
        }
        else
        {
            $adminAmount    = 0;
            $sellerAmount   = $transaction->Purchase->price;
        }

        // Configure get express checkout details call
        $getExpressCheckoutDetails = $this->service('XFA\Core:PayPal\GetExpressCheckout', \XF::options()->xfa_rmmp_paypal_testing);

        $getExpressCheckoutDetails->setToken($token);

        // Perform the call and check for errors
        $getExpressCheckoutDetails->run();

        if ($getExpressCheckoutDetails->hasErrors())
        {
            return $this->error($getExpressCheckoutDetails->getErrors());
        }

        // Configure get express checkout details call
        $doExpressCheckout = $this->service('XFA\Core:PayPal\DoExpressCheckout', \XF::options()->xfa_rmmp_paypal_testing);

        $doExpressCheckout->setExpressCheckoutDetails($getExpressCheckoutDetails->getResult());
        $doExpressCheckout->setToken($token);

        $doExpressCheckout->setItem(
            $transaction->transaction_id,
            $resource->title,
            $sellerAmount,
            $resource->xfa_rmmp_currency,
            $resource->xfa_rmmp_paypal,
            $this->buildLink('canonical:resources', $resource),
            \XF::phrase('xfa_rmmp_renewal') . ': ' . $resource->title
        );

        if ($withPercentageOnSale)
        {
            $doExpressCheckout->setFees($adminAmount, $resource->xfa_rmmp_currency, $mailForPercentageOnSale);
        }

        // Perform the call and check for errors
        $doExpressCheckout->run();

        if ($doExpressCheckout->hasErrors())
        {
            return $this->error($doExpressCheckout->getErrors());
        }

        $paymentInfo = $doExpressCheckout->getPaymentInfo();

        // Set purchase info
        $purchaseInfo = $this->setupPurchaseInfoCreate(
            $transaction->Purchase,
            $getExpressCheckoutDetails->getName(),
            $getExpressCheckoutDetails->getPhone(),
            $getExpressCheckoutDetails->getAddress(),
            $getExpressCheckoutDetails->getNote()
        )->save();

        switch ($paymentInfo['status'])
        {
            case 'Pending':
                // Update the transaction status
                $transaction->status = 'pending';
                $transaction->save();

                // Show pending error
                $viewParams = [
                    'message' => \XF::phrase('xfa_rmmp_payment_pending_error')->render()
                ];

                return $this->view('', 'message_page', $viewParams);
                break;
            case 'Completed':
            case 'Processed':
                // Update the transaction data in the database
                $transaction = $this->setupTransactionEdit($transaction, $paymentInfo)->save();

                // Validate the purchase
                $validator = $this->service('XFA\RMMarketplace:Purchase\Validate', $transaction->Purchase);
                $validator->validate();

                // Create the view params array
                $viewParams = [
                    'resource'  => $resource,
                    'type'      => 'renew'
                ];

                // Return success view
                return $this->view('', 'xfa_rmmp_payment_redirect', $viewParams);
                break;
            default:
                // Other status shouldn't happen
                throw $this->exception($this->noPermission());
                break;
        }
    }

    public function actionDownloadSelect(ParameterBag $params)
    {
        $license = $this->assertOwnedLicense($params->license_id);

        /* Check license is active */
        if (!$license->active)
        {
            return $this->error(\XF::phrase('xfa_rmmp_license_not_active'));
        }

        /* Check license url is set if url restricted */
        if ($license->Resource->DigitalProduct->require_url && $license->hasNoUrlSet())
        {
            return $this->error(\XF::phrase('xfa_rmmp_license_set_url_before_download'));
        }

        if ($this->isPost())
        {
            $resourceVersionId = $this->filter('resource_version_id', 'uint');

            /* Get version and check user can download */
            /** @var \XFRM\Finder\ResourceVersion $versionFinder */
            $versionFinder = $this->finder('XFRM:ResourceVersion');

            if ($license->Resource->DigitalProduct->duration)
            {
                $resourceVersion = $versionFinder->where('resource_version_id', $resourceVersionId)
                    ->where('resource_id', $license->resource_id)
                    ->where('release_date', '<', $license->end_date)
                    ->fetchOne();
            }
            else
            {
                $resourceVersion = $versionFinder->where('resource_version_id', $resourceVersionId)
                    ->where('resource_id', $license->resource_id)
                    ->fetchOne();
            }

            if ($resourceVersion)
            {
                /* Register capability */
                $GLOBALS['xfa_rmmp_dl_authorized'] = true;
            }

            $params->resource_version_id = $resourceVersionId;

            return $this->rerouteController('XFRM:ResourceVersion', 'download', $params);
        }
        else
        {
            /* Find versions prior to download */
            /** @var \XFRM\Finder\ResourceVersion $versionFinder */
            $versionFinder = $this->finder('XFRM:ResourceVersion');

            $versionFinder->where('resource_id', $license->resource_id);

            if ($license->Resource->DigitalProduct->duration)
            {
                $versionFinder->where('release_date', '<', $license->end_date);
            }

            $versionFinder->order('resource_version_id', 'desc');

            $viewParams = [
                'license'   => $license,
                'versions'  => $versionFinder->fetch()
            ];
            return $this->view('', 'xfa_rmmp_download_select', $viewParams);
        }
    }

    public function actionValidateCoupon(ParameterBag $params)
    {
        $license = $this->assertOwnedLicense($params->license_id);

        // Get coupon data
        $couponCode = $this->filter('coupon', 'str');
        $coupon     = $this->finder('XFA\RMMarketplace:Coupon')
            ->where('code', $couponCode)
            ->fetchOne();

        // Check coupon is usable on resource
        $usableCoupon = $coupon ? $coupon->isUsableOnResourceByUser('renewal', $license->Resource, \XF::visitor()) : false;

        if ($usableCoupon)
        {
            $validationPhrase = $coupon->getPhrase();
        }
        else
        {
            $validationPhrase = \XF::phrase('xfa_rmmp_invalid_coupon')->render();
        }

        $viewParams = [
            'usable'            => $usableCoupon,
            'usages'            => $usableCoupon ? ($coupon->UserUsages) : 0,
            'coupon'            => $coupon ? $coupon->toArray() : [],
            'validationPhrase'  => $validationPhrase
        ];

        $this->setResponseType('json');

        return $this->view('XFA\RMMarketplace:ValidateCoupon', '', $viewParams);
    }

    public function actionFreePurchase(ParameterBag $params)
    {
        $license = $this->assertOwnedLicense($params->license_id);

        $resource = $license->Resource;

        if (!$this->isPost())
        {
            throw $this->exception($this->noPermission());
        }

        // Check CGV were accepted if any
        if ($resource->isRequiringCGVValidation() && !$this->filter('xfa_agree', 'bool'))
        {
            return $this->error(\XF::phrase('xfa_rmmp_must_accept_cgv'));
        }

        // Get coupon if any
        $couponCode = $this->filter('coupon_code', 'str');
        $coupon     = \XF::finder('XFA\RMMarketplace:Coupon')
            ->where('code', $couponCode)
            ->fetchOne();

        // Check coupon is usable on resource
        $usableCoupon = $coupon ? $coupon->isUsableOnResourceByUser('renewal', $resource, \XF::visitor()) : false;
        if ($coupon && !$usableCoupon)
        {
            return $this->error(\XF::phrase('xfa_rmmp_invalid_coupon'));
        }

        // Check purchase is free
        if ($resource->DigitalProduct->getComputedRenewalPrice($coupon) > 0.0)
        {
            throw $this->exception($this->noPermission());
        }

        // Create the transaction
        $transactionCreator = $this->setupTransactionCreate($resource);
        $transactionCreator->setCompletedStatus();
        $transaction = $transactionCreator->save();

        // Create the purchase and overload price
        $purchaseCreator = $this->setupPurchaseRenewCreate($resource, $transaction, $license, $coupon);
        $purchaseCreator->setPrice(0.0);
        $purchase = $purchaseCreator->save();

        // Validate the purchase
        $validator = $this->service('XFA\RMMarketplace:Purchase\Validate', $purchase);
        $validator->validate();

        return $this->redirect($this->buildLink('resources/market-place-user/licenses'));
    }

    protected function assertEditableLicense($licenseId, array $extraWith = [])
    {
        $visitor = \XF::visitor();

        $extraWith[] = 'Resource';
        $extraWith[] = 'Resource.DigitalProduct';

        /** @var \XFA\RMMarketplace\Entity\License $license */
        $license = $this->em()->find('XFA\RMMarketplace:License', $licenseId, $extraWith);
        if (!$license)
        {
            throw $this->exception($this->notFound(\XF::phrase('xfa_rmmp_license_not_found')));
        }

        /* Check visitor is either seller of the resource or can view all sales */
        if ($license->Resource->user_id != $visitor->user_id && !$visitor->canManageAllLicenses())
        {
            throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_you_are_not_resource_owner')));
        }

        return $license;
    }

    protected function assertOwnedLicense($licenseId, array $extraWith = [])
    {
        $extraWith[] = 'Resource';
        $extraWith[] = 'Resource.DigitalProduct';

        /** @var \XFA\RMMarketplace\Entity\License $license */
        $license = $this->em()->find('XFA\RMMarketplace:License', $licenseId, $extraWith);
        if (!$license)
        {
            throw $this->exception($this->notFound(\XF::phrase('xfa_rmmp_license_not_found')));
        }

        if (!$license->isOwner())
        {
            throw $this->exception($this->noPermission());
        }

        return $license;
    }

    protected function assertRenewableLicense($licenseId, array $extraWith = [])
    {
        $extraWith[] = 'Resource';
        $extraWith[] = 'Resource.User';
        $extraWith[] = 'Resource.DigitalProduct';

        $license = $this->assertOwnedLicense($licenseId, $extraWith);

        // Check license can be renewed
        if (!$license->Resource->DigitalProduct->duration || !$license->canBeRenewed())
        {
            throw $this->exception($this->notFound(\XF::phrase('xfa_rmmp_license_cant_be_renewed')));
        }

        // Check item sales are not on hold
        if ($license->Resource->User->xfa_rmmp_user_blocked)
        {
            throw $this->exception($this->notFound(\XF::phrase('xfa_rmmp_sales_temporarily_on_hold')));
        }

        // Check item sales are not disabled
        if ($license->Resource->xfa_rmmp_sales_disabled)
        {
            throw $this->exception($this->notFound(\XF::phrase('xfa_rmmp_no_longer_for_sale')));
        }

        return $license;
    }

    protected function setupLicenseEdit(\XFA\RMMarketplace\Entity\License $license, $active, $siteUrl, $endDate)
    {
        /** @var \XFA\RMMarketplace\Service\License\Edit $editor */
        $editor = $this->service('XFA\RMMarketplace:License\Edit', $license);

        $editor->setActivationStatus($active);
        $editor->setSiteUrl($siteUrl);
        $editor->setEndDate($endDate);

        return $editor;
    }

    protected function setupTransactionCreate(\XFRM\Entity\ResourceItem $resource)
    {
        /** @var \XFA\RMMarketplace\Service\Transaction\Create $creator */
        $creator = $this->service('XFA\RMMarketplace:Transaction\Create', $resource);

        $creator->setPaypalEmail($resource->xfa_rmmp_paypal);

        return $creator;
    }

    protected function setupPurchaseRenewCreate(
        \XFRM\Entity\ResourceItem $resource,
        \XFA\RMMarketplace\Entity\Transaction $transaction,
        \XFA\RMMarketplace\Entity\License $license = null,
        \XFA\RMMarketplace\Entity\Coupon $coupon = null
    )
    {
        /** @var \XFA\RMMarketplace\Service\Purchase\Create $creator */
        $creator = $this->service('XFA\RMMarketplace:Purchase\Create', $resource, $transaction, $license);

        $creator->setType('renewal');
        $creator->setPrice($resource->DigitalProduct->getComputedRenewalPrice($coupon));
        $creator->setCoupon($coupon);

        return $creator;
    }

    protected function setupPurchaseInfoCreate(
        \XFA\RMMarketplace\Entity\Purchase $purchase,
        $name, $phone, $address, $additionalInfo
    )
    {
        /** @var \XFA\RMMarketplace\Service\Purchase\CreateInfo $creator */
        $creator = $this->service('XFA\RMMarketplace:Purchase\CreateInfo', $purchase);

        $creator->setName($name);
        $creator->setPhone($phone);
        $creator->setAddress($address);
        $creator->setAdditionalInfo($additionalInfo);

        return $creator;
    }

    protected function setupTransactionEdit(\XFA\RMMarketplace\Entity\Transaction $transaction, $paymentInfo)
    {
        /** @var \XFA\RMMarketplace\Service\Transaction\Edit $editor */
        $editor = $this->service('XFA\RMMarketplace:Transaction\Edit', $transaction);

        $editor->setStatus('completed');
        $editor->setTxnId($paymentInfo['transaction_id']);
        $editor->setFeesAmount($paymentInfo['fees_amount']);

        if ($paymentInfo['pos_transaction_id'])
        {
            $editor->setPercentageOnSaleData($paymentInfo['pos_transaction_id'], $paymentInfo['pos_amount']);
        }

        return $editor;
    }
}