<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Pub\Controller;

use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\ParameterBag;
use XF\Pub\Controller\AbstractController;
use XFA\RMMarketplace\Entity\DigitalProduct;

class Purchase extends AbstractController
{
    protected function preDispatchController($action, ParameterBag $params)
    {
        $this->assertRegistrationRequired();

        /** @var \XFRM\XF\Entity\User $visitor */
        $visitor = \XF::visitor();

        if (!$visitor->canViewResources($error))
        {
            throw $this->exception($this->noPermission($error));
        }
    }

    public function actionPurchase(ParameterBag $params)
    {
        $resource = $this->assertCustomPurchasableResource($params->resource_id, $this->getResourceViewExtraWith());

        if ($this->isPost())
        {
            // Manage redirection
            if ($resource->xfa_rmmp_accept_other && $this->filter('payment_mean', 'str') == 'manual')
            {
                return $this->rerouteController(__CLASS__, 'manual-purchase', $params);
            }
            else
            {
                return $this->rerouteController(__CLASS__, 'custom-purchase', $params);
            }
        }
        else
        {
            // In case of digital product, if redirection to renew is active check applicability and redirect
            if ($resource->xfa_rmmp_type == 'digital'
                && \XF::options()->xfa_rmmp_redirectToRenewIfOwning
                && !$this->filter('forced', 'bool'))
            {
                // Get licenses
                $renewableLicenses = $resource->DigitalProduct->getRenewableLicenses(\XF::visitor()->user_id);

                if ($renewableLicenses instanceof AbstractCollection)
                {
                    return $this->redirect(
                        $this->buildLink(
                            'resources/market-place-purchase/select-license-renewal',
                            $resource,
                            ['type' => 'custom']
                        ),
                        ''
                    );
                }
                else
                {
                    return $this->redirect(
                        $this->buildLink(
                            'resources/market-place-license/custom-renew',
                            $renewableLicenses
                        ),
                        ''
                    );
                }
            }

            $viewParams = [
                'resource' => $resource
            ];

            // Check if the user already has an expired license
            if ($resource->xfa_rmmp_type == 'digital' && $resource->DigitalProduct->ExpiredLicense && $resource->DigitalProduct->allow_expired_renewal)
            {
                $viewParams['hasExpiredLicense'] = 1;
            }

            return $this->view('', 'xfa_rmmp_purchase_custom', $viewParams);
        }
    }

    public function actionManualPurchase(ParameterBag $params)
    {
        $resource = $this->assertPurchasableResource($params->resource_id, $this->getResourceViewExtraWith());

        if ($this->isPost())
        {
            /* Check CGV were accepted if any */
            if ($resource->isRequiringCGVValidation() && !$this->filter('xfa_agree', 'bool'))
            {
                return $this->error(\XF::phrase('xfa_rmmp_must_accept_cgv'));
            }

            // Get coupon if any
            $couponCode = $this->filter('coupon_code', 'str');
            $coupon     = \XF::finder('XFA\RMMarketplace:Coupon')
                ->where('code', $couponCode)
                ->fetchOne();

            // Check coupon is usable on resource
            $usableCoupon = $coupon ? $coupon->isUsableOnResourceByUser('purchase', $resource, \XF::visitor()) : false;
            if ($couponCode && !$usableCoupon)
            {
                return $this->error(\XF::phrase('xfa_rmmp_invalid_coupon'));
            }

            // Filter data
            $inputData = $this->filter([
                'first_name'        => 'str',
                'last_name'         => 'str',
                'phone'             => 'str',
                'street'            => 'str',
                'street2'           => 'str',
                'city'              => 'str',
                'zip'               => 'str',
                'state'             => 'str',
                'country'           => 'str',
                'additional_info'   => 'str',
                'quantity'          => 'uint'
            ]);

            // Check additional data if physical
            if ($resource->xfa_rmmp_type == 'physical')
            {
                // Check info fields
                if (empty($inputData['first_name'])
                    || empty($inputData['last_name'])
                    || empty($inputData['street'])
                    || empty($inputData['city'])
                    || empty($inputData['zip'])
                    || empty($inputData['country']))
                {
                    return $this->error(\XF::phrase('xfa_rmmp_must_set_info_fields'));
                }
            }

            // Check quantity if needed
            if ($resource->canBePurchasedInMultipleQty())
            {
                $maxQty = $resource->getPurchasableQty();
                if ($maxQty > 0 && $inputData['quantity'] > $maxQty)
                {
                    return $this->error(\XF::phrase('xfa_rmmp_not_allowed_purchase_that_quantity'));
                }
            }
            else
            {
                // Default quantity to 1
                $inputData['quantity'] = 1;
            }            
        
		    // Compute shipping if any
		    if ($resource->xfa_rmmp_type == 'physical' && $resource->PhysicalProduct->shipping)
		    {
		        $shipping = round($this->filter('shipping', 'float'), 2);
		    }
		    else
		    {
		        $shipping = 0;
		    }   

            // Create the transaction
            $transactionCreator = $this->setupPurchaseTransactionCreate($resource, $shipping);
            $transactionCreator->setPendingManualStatus();
            $transaction = $transactionCreator->save();

            // Create the purchase
            $purchaseCreator = $this->setupPurchaseCreate($resource, $transaction, $inputData['quantity'], $shipping);
            $purchase = $purchaseCreator->save();

            // Create inactive license(s) if digital product
            if ($resource->xfa_rmmp_type == 'digital')
            {
                for ($i = 0; $i < $inputData['quantity']; $i++)
                {
                    $this->setupLicenseCreate($resource, $purchase)->save();
                }
            }
            else
            {
                $license = null;
            }

            // Store additional data if physical product
            if ($resource->xfa_rmmp_type == 'physical')
            {
                $name = $inputData['last_name'] . ' ' . $inputData['first_name'];

                $address = $inputData['street'];

                if (!empty($inputData['street2']))
                {
                    $address .= "\r\n" . $inputData['street2'];
                }

                $address .= "\r\n" . $inputData['zip'] . ' ' . $inputData['city'];

                if (!empty($inputData['state']))
                {
                    $address .= "\r\n" . $inputData['state'];
                }

                $address .= "\r\n" . $inputData['country'];

                $this->setupPurchaseInfoCreate(
                    $transaction->Purchase,
                    $name,
                    $inputData['phone'],
                    $address,
                    $inputData['additional_info']
                )->save();
            }

            // Create the conversation
            $conversation = $this->setupPurchaseConversationCreate($resource, $purchase, ($resource->Seller ? $resource->Seller->activate_invoicing : false))->save();

            return $this->redirect($this->buildLink('conversations', $conversation));
        }
        else
        {
            $viewParams = [
                'resource' => $resource
            ];
            return $this->view('', 'xfa_rmmp_other_purchase_form', $viewParams);
        }
    }

    public function actionFreePurchase(ParameterBag $params)
    {
        $resource = $this->assertPurchasableResource($params->resource_id, $this->getResourceViewExtraWith());

        if (!$this->isPost())
        {
            throw $this->exception($this->noPermission());
        }

        // Check CGV were accepted if any
        if ($resource->isRequiringCGVValidation() && !$this->filter('xfa_agree', 'bool'))
        {
            return $this->error(\XF::phrase('xfa_rmmp_must_accept_cgv'));
        }

        // Get coupon if any
        $couponCode = $this->filter('coupon_code', 'str');
        $coupon     = \XF::finder('XFA\RMMarketplace:Coupon')
            ->where('code', $couponCode)
            ->fetchOne();

        // Check coupon is usable on resource
        $usableCoupon = $coupon ? $coupon->isUsableOnResourceByUser('purchase', $resource, \XF::visitor()) : false;
        if ($coupon && !$usableCoupon)
        {
            return $this->error(\XF::phrase('xfa_rmmp_invalid_coupon'));
        }

        // Filter data
        $inputData = $this->filter([
            'first_name'        => 'str',
            'last_name'         => 'str',
            'phone'             => 'str',
            'street'            => 'str',
            'street2'           => 'str',
            'city'              => 'str',
            'zip'               => 'str',
            'state'             => 'str',
            'country'           => 'str',
            'additional_info'   => 'str',
            'quantity'          => 'uint'
        ]);

        // Check additional data if physical
        if ($resource->xfa_rmmp_type == 'physical')
        {
            // Check info fields
            if (empty($inputData['first_name'])
                || empty($inputData['last_name'])
                || empty($inputData['street'])
                || empty($inputData['city'])
                || empty($inputData['zip'])
                || empty($inputData['country']))
            {
                return $this->error(\XF::phrase('xfa_rmmp_must_set_info_fields'));
            }
        }

        // Check quantity if needed
        if ($resource->canBePurchasedInMultipleQty())
        {
            $maxQty = $resource->getPurchasableQty();
            if ($maxQty > 0 && $inputData['quantity'] > $maxQty)
            {
                return $this->error(\XF::phrase('xfa_rmmp_not_allowed_purchase_that_quantity'));
            }
        }
        else
        {
            // Default quantity to 1
            $inputData['quantity'] = 1;
        }

        // Compute shipping if any
        if ($resource->xfa_rmmp_type == 'physical' && $resource->PhysicalProduct->shipping)
        {
            $shipping = round($this->filter('shipping', 'float'), 2);
        }
        else
        {
            $shipping = 0.0;
        }

        // Check purchase is free
        $totalPrice = $resource->getComputedResourcePrice($inputData['quantity'], $coupon) + $shipping;
        if ($totalPrice > 0.0)
        {
            throw $this->exception($this->noPermission());
        }

        // Create the transaction
        $transactionCreator = $this->setupPurchaseTransactionCreate($resource, $shipping);
        $transactionCreator->setCompletedStatus();
        $transaction = $transactionCreator->save();

        // Create the purchase and set price to 0
        $purchaseCreator = $this->setupPurchaseCreate($resource, $transaction, $inputData['quantity'], $shipping, $coupon);
        $purchaseCreator->setPrice(0.0);
        $purchase = $purchaseCreator->save();

        // Create inactive license(s) if digital product
        if ($resource->xfa_rmmp_type == 'digital')
        {
            for ($i = 0; $i < $inputData['quantity']; $i++)
            {
                $this->setupLicenseCreate($resource, $purchase)->save();
            }
        }
        else
        {
            $license = null;
        }

        // Store additional data if physical product
        if ($resource->xfa_rmmp_type == 'physical')
        {
            $name = $inputData['last_name'] . ' ' . $inputData['first_name'];

            $address = $inputData['street'];

            if (!empty($inputData['street2']))
            {
                $address .= "\r\n" . $inputData['street2'];
            }

            $address .= "\r\n" . $inputData['zip'] . ' ' . $inputData['city'];

            if (!empty($inputData['state']))
            {
                $address .= "\r\n" . $inputData['state'];
            }

            $address .= "\r\n" . $inputData['country'];

            $this->setupPurchaseInfoCreate(
                $transaction->Purchase,
                $name,
                $inputData['phone'],
                $address,
                $inputData['additional_info']
            )->save();
        }

        // Validate the purchase
        $validator = $this->service('XFA\RMMarketplace:Purchase\Validate', $purchase);
        $validator->validate();

        return $this->redirect($this->buildLink('resources/market-place-user/purchases'));
    }

    public function actionCustomPurchase(ParameterBag $params)
    {
        $resource = $this->assertCustomPurchasableResource($params->resource_id, $this->getResourceViewExtraWith());

        // Check CGV were accepted if any
        if ($resource->isRequiringCGVValidation() && !$this->filter('xfa_agree', 'bool'))
        {
            return $this->error(\XF::phrase('xfa_rmmp_must_accept_cgv'));
        }

        // Get coupon if any
        $couponCode = $this->filter('coupon_code', 'str');
        $coupon     = \XF::finder('XFA\RMMarketplace:Coupon')
            ->where('code', $couponCode)
            ->fetchOne();

        // Check coupon is usable on resource
        $usableCoupon = $coupon ? $coupon->isUsableOnResourceByUser('purchase', $resource, \XF::visitor()) : false;
        if ($couponCode && !$usableCoupon)
        {
            $error = \XF::phrase('xfa_rmmp_invalid_coupon');
            return false;
        }

        // Check quantity if needed
        $quantity = $this->filter('quantity', 'uint', 1);
        if ($resource->canBePurchasedInMultipleQty())
        {
            $maxQty = $resource->getPurchasableQty();
            if ($maxQty > 0 && $quantity > $maxQty)
            {
                return $this->error(\XF::phrase('xfa_rmmp_not_allowed_purchase_that_quantity'));
            }
        }
        
        // Compute shipping if any
        if ($resource->xfa_rmmp_type == 'physical' && $resource->PhysicalProduct->shipping)
        {
	        $shipping = round($this->filter('shipping', 'float'), 2);
        }
        else
        {
	        $shipping = 0;
        }     

        $transaction = $this->setupPurchaseTransactionCreate($resource, $shipping)->save();

        // Create the purchase
        $purchase = $this->setupPurchaseCreate($resource, $transaction, $quantity, $shipping, $coupon)->save();

        // Create inactive license(s) if digital product
        if ($resource->xfa_rmmp_type == 'digital')
        {
            for ($i = 0; $i < $quantity; $i++)
            {
                $this->setupLicenseCreate($resource, $purchase)->save();
            }
        }
        else
        {
            $license = null;
        }

        // Check if percentage on sales for category or globally
        if ($resource->bypassPercentageOnSale())
        {
            $withPercentageOnSale       = 0;
            $percentageOnSale           = 0;
            $mailForPercentageOnSale    = '';	        
        }
        else
        {
	        if ($resource->Category->xfa_rmmp_sale_percentage > 0.00)
	        {
	            $withPercentageOnSale       = 1;
	            $percentageOnSale           = $resource->Category->xfa_rmmp_sale_percentage;
	            $mailForPercentageOnSale    = $resource->Category->xfa_rmmp_sale_percentage_paypal;
	        }
	        else if (\XF::options()->xfa_rmmp_sale_percentage > 0.00)
	        {
	            $withPercentageOnSale       = 1;
	            $percentageOnSale           = \XF::options()->xfa_rmmp_sale_percentage;
	            $mailForPercentageOnSale    = \XF::options()->xfa_rmmp_paypal;
	        }
	        else
	        {
	            $withPercentageOnSale       = 0;
	            $percentageOnSale           = 0;
	            $mailForPercentageOnSale    = '';
	        }
	    }

        // Compute amounts based on that
        if ($withPercentageOnSale)
        {
            $adminAmount    = round(($purchase->price * $percentageOnSale / 100), 2);
            $sellerAmount   = $purchase->price - $adminAmount;
        }
        else
        {
            $adminAmount    = 0;
            $sellerAmount   = $purchase->price;
        }   

        /* Configure set express checkout call */
        $setExpressCheckout = $this->service('XFA\Core:PayPal\SetExpressCheckout', \XF::options()->xfa_rmmp_paypal_testing);

        $setExpressCheckout->setUrls(
            $this->buildLink('canonical:resources/market-place-purchase/custom-purchase-done', $purchase),
            $this->buildLink('canonical:resources', $resource)
        );

        $setExpressCheckout->setBuyerEmail(\XF::visitor()->email);

        $setExpressCheckout->setItem(
            $transaction->transaction_id,
            $resource->title . (($quantity > 1) ? ' - ' . \XF::phrase('xfa_rmmp_qty')->render() . ': ' . $quantity . '' : ''),
            $sellerAmount,
            $resource->xfa_rmmp_currency,
            $resource->getMarketplacePaymentEmail(),
            $this->buildLink('canonical:resources', $resource)
        );

        if ($withPercentageOnSale)
        {
            $setExpressCheckout->setFees($adminAmount, $resource->xfa_rmmp_currency, $mailForPercentageOnSale);
        }

		if ($shipping > 0)
		{
			$setExpressCheckout->setShipping($shipping);
		}

        /* Perform the call and check for errors */
        $setExpressCheckout->run();

        if ($setExpressCheckout->hasErrors())
        {
            return $this->error($setExpressCheckout->getErrors());
        }

        /* Update transaction and redirect to paypal */
        $transaction->token = $setExpressCheckout->getToken();
        $transaction->save();

        $viewParams = [
            'resource'  => $resource,
            'redirect'  => $setExpressCheckout->getRedirectUrl()
        ];

        return $this->view('', 'xfa_core_paypal_express_checkout_redirect', $viewParams);
    }

    public function actionCustomPurchaseDone(ParameterBag $params)
    {
        $resource = $this->assertCustomPurchasableResource($params->resource_id, $this->getResourceViewExtraWith());

        // At this stage, if no errors were encountered we need to decrement quantity for physical products
        if ($resource->xfa_rmmp_type == 'physical' && $resource->PhysicalProduct->quantity)
        {
            $resource->PhysicalProduct->quantity--;
            $resource->PhysicalProduct->save();
        }

        // Check provided token correspond to an existing transaction
        $token = $this->filter('token', 'str');

        $transaction = \XF::finder('XFA\RMMarketplace:Transaction')->where('token', $token)->fetchOne();
        if (!$transaction)
        {
            return $this->error(\XF::phrase('xfa_rmmp_transaction_doesnt_exist'));
        }

        // Check transaction and resource are matching
        if (!$transaction->Purchase || $transaction->Purchase->resource_id != $resource->resource_id)
        {
            return $this->error(\XF::phrase('xfa_rmmp_purchase_and_resource_do_not_match'));
        }

        // Check if percentage on sales for category or globally
        if ($resource->bypassPercentageOnSale())
        {
            $withPercentageOnSale       = 0;
            $percentageOnSale           = 0;
            $mailForPercentageOnSale    = '';	        
        }
        else
        {        
	        if ($resource->Category->xfa_rmmp_sale_percentage > 0.00)
	        {
	            $withPercentageOnSale       = 1;
	            $percentageOnSale           = $resource->Category->xfa_rmmp_sale_percentage;
	            $mailForPercentageOnSale    = $resource->Category->xfa_rmmp_sale_percentage_paypal;
	        }
	        else if (\XF::options()->xfa_rmmp_sale_percentage > 0.00)
	        {
	            $withPercentageOnSale       = 1;
	            $percentageOnSale           = \XF::options()->xfa_rmmp_sale_percentage;
	            $mailForPercentageOnSale    = \XF::options()->xfa_rmmp_paypal;
	        }
	        else
	        {
	            $withPercentageOnSale       = 0;
	            $percentageOnSale           = 0;
	            $mailForPercentageOnSale    = '';
	        }
	    }

        // Compute amounts based on that
        if ($withPercentageOnSale)
        {
            $adminAmount    = round(($transaction->Purchase->price * $percentageOnSale / 100), 2);
            $sellerAmount   = $transaction->Purchase->price - $adminAmount;
        }
        else
        {
            $adminAmount    = 0;
            $sellerAmount   = $transaction->Purchase->price;
        }

        // Configure get express checkout details call
        $getExpressCheckoutDetails = $this->service('XFA\Core:PayPal\GetExpressCheckout', \XF::options()->xfa_rmmp_paypal_testing);

        $getExpressCheckoutDetails->setToken($token);

        // Perform the call and check for errors
        $getExpressCheckoutDetails->run();

        if ($getExpressCheckoutDetails->hasErrors())
        {
            // Reincrement count for physical product as purchase didn't go through
            $resource->PhysicalProduct->quantity++;
            $resource->PhysicalProduct->save();

            return $this->error($getExpressCheckoutDetails->getErrors());
        }

        // Configure get express checkout details call
        $doExpressCheckout = $this->service('XFA\Core:PayPal\DoExpressCheckout', \XF::options()->xfa_rmmp_paypal_testing);

        $doExpressCheckout->setExpressCheckoutDetails($getExpressCheckoutDetails->getResult());
        $doExpressCheckout->setToken($token);

        $doExpressCheckout->setItem(
            $transaction->transaction_id,
            $resource->title . (($transaction->Purchase->quantity > 1) ? ' - ' . \XF::phrase('xfa_rmmp_qty')->render() . ': ' . $transaction->Purchase->quantity . ')' : ''),
            $sellerAmount,
            $resource->xfa_rmmp_currency,
            $resource->getMarketplacePaymentEmail(),
            $this->buildLink('canonical:resources', $resource)
        );

        if ($withPercentageOnSale)
        {
            $doExpressCheckout->setFees($adminAmount, $resource->xfa_rmmp_currency, $mailForPercentageOnSale);
        }

		if ($transaction->shipping > 0.0)
		{
			$doExpressCheckout->setShipping($transaction->shipping);
		}

        // Perform the call and check for errors
        $doExpressCheckout->run();

        if ($doExpressCheckout->hasErrors())
        {
            // Reincrement count for physical product as purchase didn't go through
            if ($resource->xfa_rmmp_type == 'physical')
            {
                $resource->PhysicalProduct->quantity++;
                $resource->PhysicalProduct->save();
            }

            return $this->error($doExpressCheckout->getErrors());
        }

        $paymentInfo = $doExpressCheckout->getPaymentInfo();

        // Set purchase info
        $purchaseInfo = $this->setupPurchaseInfoCreate(
            $transaction->Purchase,
            $getExpressCheckoutDetails->getName(),
            $getExpressCheckoutDetails->getPhone(),
            $getExpressCheckoutDetails->getAddress(\XF::options()->xfa_rmmp_usAddressFormat),
            $getExpressCheckoutDetails->getNote()
        )->save();

        switch ($paymentInfo['status'])
        {
            case 'Pending':
                // Update the transaction status
                $transaction->status = 'pending';
                $transaction->save();

                // Show pending error
                $viewParams = [
                    'message' => \XF::phrase('xfa_rmmp_payment_pending_error')->render()
                ];

                return $this->view('', 'message_page', $viewParams);
                break;
            case 'Completed':
            case 'Processed':
                // Update the transaction data in the database
                $transaction = $this->setupPurchaseTransactionEdit($transaction, $paymentInfo)->save();

                // Validate the purchase
                $validator = $this->service('XFA\RMMarketplace:Purchase\Validate', $transaction->Purchase);
                $validator->validate();

                // Create the view params array
                $viewParams = [
                    'resource'  => $resource,
                    'type'      => 'purchase'
                ];

                /* Return success view */
                return $this->view('', 'xfa_rmmp_payment_redirect', $viewParams);
                break;
            default:
                /* Other status shouldn't happen */
                throw $this->exception($this->noPermission());
                break;
        }
    }

    public function actionShipmentData(ParameterBag $params)
    {
        $extraWith = $this->getResourceViewExtraWith();
        $extraWith[] = 'PhysicalProduct';

        /** @var \XFRM\Entity\ResourceItem $resource */
        $resource = $this->em()->find('XFRM:ResourceItem', $params->resource_id, $extraWith);
        if (!$resource || $resource->xfa_rmmp_type != 'physical')
        {
            throw $this->exception($this->notFound(\XF::phrase('xfrm_requested_resource_not_found')));
        }

        if (!$resource->canView($error))
        {
            throw $this->exception($this->noPermission($error));
        }

        /* Check resource is for sale */
        if (!$resource->isForSaleInMarketplace())
        {
            return $this->error(\XF::phrase('xfa_rmmp_resource_is_not_paid'));
        }

        /** @var \XFA\RMMarketplace\Entity\Purchase $purchase */
        $purchase = $this->em()->find('XFA\RMMarketplace:Purchase', $params->purchase_id, ['Resource','Resource.Category']);
        if (!$purchase)
        {
            throw $this->exception($this->notFound(\XF::phrase('xfa_rmmp_purchase_not_found')));
        }

        /* Check rights */
        if (!$purchase->canView($error))
        {
            throw $this->exception($this->noPermission($error));
        }

        $viewParams = [
            'purchase' => $purchase
        ];

        return $this->view('', 'xfa_rmmp_shipment_data', $viewParams);
    }

    public function actionInvoice(ParameterBag $params)
    {
        $extraWith = $this->getResourceViewExtraWith();
        $extraWith[] = 'Seller';

        /** @var \XFRM\Entity\ResourceItem $resource */
        $resource = $this->em()->find('XFRM:ResourceItem', $params->resource_id, $extraWith);
        if (!$resource)
        {
            throw $this->exception($this->notFound(\XF::phrase('xfrm_requested_resource_not_found')));
        }

        if (!$resource->canView($error))
        {
            throw $this->exception($this->noPermission($error));
        }

        /* Check resource is for sale */
        if (!$resource->isForSaleInMarketplace())
        {
            return $this->error(\XF::phrase('xfa_rmmp_resource_is_not_paid'));
        }

        /** @var \XFA\RMMarketplace\Entity\Purchase $purchase */
        $purchase = $this->em()->find('XFA\RMMarketplace:Purchase', $params->purchase_id, ['Resource','Resource.Category','PurchaseInfo']);
        if (!$purchase)
        {
            throw $this->exception($this->notFound(\XF::phrase('xfa_rmmp_purchase_not_found')));
        }

        /* Check rights */
        if (!$purchase->canView($error))
        {
            throw $this->exception($this->noPermission($error));
        }

        $viewParams = [
            'purchase' => $purchase
        ];

        $this->setResponseType('raw');

        return $this->view('XFA\RMMarketplace:Invoice', '', $viewParams);
    }

    public function actionValidate(ParameterBag $params)
    {
        $purchase = $this->assertEditablePurchase($params->purchase_id, ['Resource','Resource.Category','PurchaseInfo']);

        /* Can't validate a non pending purchase */
        if ($purchase->status != 'pending')
        {
            throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_can_only_validate_pending_purchase')));
        }

        if ($this->isPost())
        {
            /* Update the transaction data in the database */
            $paymentInfo = [
                'transaction_id'    => '',
                'fees_amount'       => 0

            ];
            $transaction = $this->setupPurchaseTransactionEdit($purchase->Transaction, $paymentInfo)->save();

            /* Validate the purchase */
            $validator = $this->service('XFA\RMMarketplace:Purchase\Validate', $transaction->Purchase);
            $validator->validate();

            return $this->redirect($this->buildLink('resources/market-place-dashboard/view-sales'));
        }
        else
        {
            $viewParams = [
                'purchase' => $purchase
            ];

            return $this->view('', 'xfa_rmmp_validate_purchase', $viewParams);
        }
    }

    public function actionRefund(ParameterBag $params)
    {
        $purchase = $this->assertEditablePurchase($params->purchase_id, ['Resource','Resource.Category','PurchaseInfo']);

        /* Can't refund a non validated purchase */
        if ($purchase->status != 'validated')
        {
            throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_cant_refund_non_validated_purchase')));
        }

        if ($this->isPost())
        {
            /** @var \XFA\RMMarketplace\Service\Purchase\Refund $refunder */
            $refunder = $this->service('XFA\RMMarketplace:Purchase\Refund', $purchase)->refund();

            return $this->redirect($this->buildLink('resources/market-place-dashboard/view-sales'));
        }
        else
        {
            $viewParams = [
                'purchase' => $purchase
            ];

            return $this->view('', 'xfa_rmmp_refund_purchase', $viewParams);
        }
    }

    public function actionDelete(ParameterBag $params)
    {
        $purchase = $this->assertEditablePurchase($params->purchase_id, ['Resource','Resource.Category','PurchaseInfo']);

        if ($this->isPost())
        {
            /** @var \XFA\RMMarketplace\Service\Purchase\Delete $deleter */
            $deleter = $this->service('XFA\RMMarketplace:Purchase\Delete', $purchase)->delete();

            return $this->redirect($this->buildLink('resources/market-place-dashboard/view-sales'));
        }
        else
        {
            $viewParams = [
                'purchase' => $purchase
            ];

            return $this->view('', 'xfa_rmmp_delete_purchase', $viewParams);
        }
    }

    public function actionShowShipmentInfo(ParameterBag $params)
    {
        $purchase = $this->assertEditablePurchase($params->purchase_id, ['Resource','Resource.Category','PurchaseInfo']);

        $viewParams = [
            'purchase' => $purchase
        ];

        return $this->view('', 'xfa_rmmp_shipment_info', $viewParams);
    }

    public function actionEditShipmentStatus(ParameterBag $params)
    {
        $purchase = $this->assertEditablePurchase($params->purchase_id, ['Resource','Resource.Category','PurchaseInfo']);

        if ($this->isPost())
        {
            /* Update the Purchase Info data in the database */
            $shipmentInfo = [
                'shipment_status'   => $this->filter('shipment_status', 'str'),
                'shipment_data'     => $this->filter('shipment_data', 'str')
            ];

            $purchase = $this->setupShipmentStatusEdit($purchase, $shipmentInfo)->save();

            return $this->redirect($this->buildLink('resources/market-place-dashboard/view-sales'));
        }
        else
        {
            $viewParams = [
                'purchase' => $purchase
            ];

            return $this->view('', 'xfa_rmmp_edit_shipment_status', $viewParams);
        }
    }

    public function actionPurchaseXf(ParameterBag $params)
    {
        $resource = $this->assertPurchasableResource($params->resource_id, $this->getResourceViewExtraWith());

        // In case of digital product, if redirection to renew is active check applicability and redirect
        if ($resource->xfa_rmmp_type == 'digital'
            && \XF::options()->xfa_rmmp_redirectToRenewIfOwning
            && !$this->filter('forced', 'bool'))
        {
            // Get licenses
            $renewableLicenses = $resource->DigitalProduct->getRenewableLicenses(\XF::visitor()->user_id);

            if ($renewableLicenses instanceof AbstractCollection)
            {
                return $this->redirect(
                    $this->buildLink(
                        'resources/market-place-purchase/select-license-renewal',
                        $resource,
                        ['type' => 'xf']
                    ),
                    ''
                );
            }
            else
            {
                return $this->redirect(
                    $this->buildLink(
                        'resources/market-place-license/renew-xf',
                        $renewableLicenses
                    ),
                    ''
                );
            }
        }

        // Check purchasability
        $purchasable = $this->em()->find('XF:Purchasable', 'xfa_rmmp_purchase', 'AddOn');
        if (!$purchasable || !$purchasable->isActive())
        {
            return $this->message(\XF::phrase('xfa_rmmp_no_purchase_cant_be_done_right_now'));
        }

        // Check we have xF payment method activated
        if (\XF::options()->xfa_rmmp_mode != 'xf')
        {
            throw $this->exception($this->noPermission());
        }
        
        // Get payment profiles depending on configuration
        $profiles = null;
        switch ($resource->Category->xfa_rmmp_conf)
        {
	        // User
	        case 1:
				$profiles = $this->finder('XF:PaymentProfile')
									->where('provider_id', \XF::options()->xfa_rmmp_paymentProviderIds)
									->where('xfa_rmmp_user_id', $resource->user_id)
									->where('active', 1)
									->fetch();
				break;
			// Category			
			case 2:
				$profiles = $this->finder('XF:PaymentProfile')
									->where('payment_profile_id', explode(',', $resource->Category->xfa_rmmp_payment_data))
									->where('active', 1)
									->fetch();
				break;
			// Forum
			case 3:
				$profiles = $this->finder('XF:PaymentProfile')
									->where('payment_profile_id', \XF::options()->xfa_rmmp_paymentProfileIds)
									->where('active', 1)
									->fetch();
				break;
        }
        
        if (!count($profiles))
        {
            throw $this->exception($this->noPermission());
        }
                
        $viewParams = [
            'resource'  	=> $resource,
            'profiles'  	=> $profiles
        ];

        // Check if the user already has an expired license
        if ($resource->xfa_rmmp_type == 'digital' && $resource->DigitalProduct->ExpiredLicense && $resource->DigitalProduct->allow_expired_renewal)
        {
            $viewParams['hasExpiredLicense'] = 1;
        }

        return $this->view('', 'xfa_rmmp_purchase_xf', $viewParams);
    }

    public function actionValidateCoupon(ParameterBag $params)
    {
        $resource = $this->assertPurchasableResource($params->resource_id, $this->getResourceViewExtraWith());

        // Get coupon data
        $couponCode = $this->filter('coupon', 'str');
        $coupon     = $this->finder('XFA\RMMarketplace:Coupon')
                            ->where('code', $couponCode)
                            ->fetchOne();

        // Check coupon is usable on resource
        $usableCoupon = $coupon ? $coupon->isUsableOnResourceByUser('purchase', $resource, \XF::visitor()) : false;

        if ($usableCoupon)
        {
            $validationPhrase = $coupon->getPhrase();
        }
        else
        {
            $validationPhrase = \XF::phrase('xfa_rmmp_invalid_coupon')->render();
        }

        $viewParams = [
            'usable'            => $usableCoupon,
            'usages'            => $usableCoupon ? ($coupon->UserUsages) : 0,
            'coupon'            => $coupon ? $coupon->toArray() : [],
            'validationPhrase'  => $validationPhrase
        ];

        $this->setResponseType('json');

        return $this->view('XFA\RMMarketplace:ValidateCoupon', '', $viewParams);
    }

    public function actionSelectLicenseRenewal(ParameterBag $params)
    {
        $resource = $this->assertPurchasableResource($params->resource_id, $this->getResourceViewExtraWith());

        // In case of digital product, if redirection to renew is active check applicability and redirect
        if ($resource->xfa_rmmp_type != 'digital'
            || !\XF::options()->xfa_rmmp_redirectToRenewIfOwning)
        {
            throw $this->exception($this->noPermission());
        }

        // Get licenses
        $renewableLicenses = $resource->DigitalProduct->getRenewableLicenses(\XF::visitor()->user_id);

        $viewParams = [
            'resource'  => $resource,
            'type'      => $this->filter('type', 'str'),
            'licenses'  => $renewableLicenses
        ];

        return $this->view('', 'xfa_rmmp_select_license_renewal', $viewParams);
    }

    protected function assertPurchasableResource($resourceId, array $extraWith = [])
    {
        $visitor = \XF::visitor();

        $extraWith[] = 'User';
        $extraWith[] = 'Category';
        $extraWith[] = 'Category.Permissions|' . $visitor->permission_combination_id;
        $extraWith[] = 'Description';
        $extraWith[] = 'CurrentVersion';
        $extraWith[] = 'Discussion';
        $extraWith[] = 'Discussion.Forum';
        $extraWith[] = 'Discussion.Forum.Node';
        $extraWith[] = 'Discussion.Forum.Node.Permissions|' . $visitor->permission_combination_id;

        /** @var \XFRM\Entity\ResourceItem $resource */
        $resource = $this->em()->find('XFRM:ResourceItem', $resourceId, $extraWith);
        if (!$resource)
        {
            throw $this->exception($this->notFound(\XF::phrase('xfrm_requested_resource_not_found')));
        }

        if (!$resource->canView($error))
        {
            throw $this->exception($this->noPermission($error));
        }

        // Check resource is for sale and purchasable
        if (!$resource->isPurchasable() && !$resource->isPurchasableForFriend())
        {
            throw $this->exception($this->noPermission($error));
        }

        // Check item sales are not on hold
        if ($resource->User->xfa_rmmp_user_blocked)
        {
            throw $this->exception($this->notFound(\XF::phrase('xfa_rmmp_sales_temporarily_on_hold')));
        }

        // Check item sales are not disabled
        if ($resource->xfa_rmmp_sales_disabled)
        {
            throw $this->exception($this->notFound(\XF::phrase('xfa_rmmp_no_longer_for_sale')));
        }

        return $resource;
    }
    
    protected function assertCustomPurchasableResource($resourceId, array $extraWith = [])
    {
	    $resource = $this->assertPurchasableResource($resourceId, $extraWith);
	    
        // Check we have custom payment method activated
        if (\XF::options()->xfa_rmmp_mode != 'custom')
        {
            throw $this->exception($this->noPermission());
        }

        // Check item sales are not on hold
        if ($resource->User->xfa_rmmp_user_blocked)
        {
            throw $this->exception($this->notFound(\XF::phrase('xfa_rmmp_sales_temporarily_on_hold')));
        }

        // Check item sales are not disabled
        if ($resource->xfa_rmmp_sales_disabled)
        {
            throw $this->exception($this->notFound(\XF::phrase('xfa_rmmp_no_longer_for_sale')));
        }
	    
	    return $resource;
	}    

    protected function assertEditablePurchase($purchaseId, array $extraWith = [])
    {
        $visitor = \XF::visitor();

        /** @var \XFA\RMMarketplace\Entity\Purchase $purchase */
        $purchase = $this->em()->find('XFA\RMMarketplace:Purchase', $purchaseId, $extraWith);
        if (!$purchase)
        {
            throw $this->exception($this->notFound(\XF::phrase('xfa_rmmp_purchase_not_found')));
        }

        /* Check visitor is either seller of the resource or can view all sales */
        if ($purchase->Resource->user_id != $visitor->user_id && !$visitor->canViewAllSales())
        {
            throw $this->exception($this->noPermission(\XF::phrase('xfa_rmmp_you_are_not_resource_owner')));
        }

        return $purchase;
    }

    protected function getResourceViewExtraWith()
    {
        $extraWith = ['Featured'];
        $userId = \XF::visitor()->user_id;
        if ($userId)
        {
            $extraWith[] = 'Watch|' . $userId;
            $extraWith[] = 'CurrentVersion.Downloads|' . $userId;
        }

        return $extraWith;
    }

    protected function setupPurchaseTransactionCreate(\XFRM\Entity\ResourceItem $resource, $shipping)
    {
        /** @var \XFA\RMMarketplace\Service\Transaction\Create $creator */
        $creator = $this->service('XFA\RMMarketplace:Transaction\Create', $resource);

        $creator->setPaypalEmail($resource->getMarketplacePaymentEmail());
        $creator->setShipping($shipping);

        return $creator;
    }

    protected function setupLicenseCreate(\XFRM\Entity\ResourceItem $resource, \XFA\RMMarketplace\Entity\Purchase $purchase)
    {
        /** @var \XFA\RMMarketplace\Service\License\Create $creator */
        $creator = $this->service('XFA\RMMarketplace:License\Create', $resource);

        $creator->setPurchaseId($purchase->purchase_id);

        if ($resource->DigitalProduct->duration)
        {
            $creator->setEndDateWithDuration($resource->DigitalProduct->getDurationString());
        }
        
        if ($purchase->for_user_id != 0)
        {
            $creator->setOwner($purchase->ForUser);
        }        

        return $creator;
    }

    protected function setupPurchaseCreate(
        \XFRM\Entity\ResourceItem $resource,
        \XFA\RMMarketplace\Entity\Transaction $transaction,
        $quantity,
        $shipping,
        \XFA\RMMarketplace\Entity\Coupon $coupon = null
    )
    {
        /** @var \XFA\RMMarketplace\Service\Purchase\Create $creator */
        $creator = $this->service('XFA\RMMarketplace:Purchase\Create', $resource, $transaction);

        $creator->setQuantity($quantity);
        $creator->setPrice($resource->getComputedResourcePrice($quantity, $coupon) + $shipping);

        if ($coupon)
        {
            $creator->setCoupon($coupon);
        }

        if ($resource->xfa_rmmp_type == 'digital' && $resource->isPurchasableForFriend())
        {
            $username = $this->filter('for_username', 'str');

            /** @var \XF\Repository\User $userRepo */
            $userRepo = $this->repository('XF:User');
            $user = $userRepo->getUserByNameOrEmail($username);

            if ($user)
            {
                $creator->setForUser($user);
            }
            else if (!$resource->isPurchasable())
            {
	            // Resource must be purchased for a friend if not purchasable for oneself
	            throw $this->exception($this->noPermission());	            
            }
        }

        return $creator;
    }

    protected function setupPurchaseWithInfoCreate(
        \XFRM\Entity\ResourceItem $resource,
        \XFA\RMMarketplace\Entity\Transaction $transaction,
        $shipping,
        $inputData,
        \XFA\RMMarketplace\Entity\Coupon $coupon = null
    )
    {
        // Compute the price
        $price = $resource->getComputedResourcePrice($inputData['quantity'], $coupon);
        $price += $shipping;

        /** @var \XFA\RMMarketplace\Service\Purchase\CreateWithInfo $creator */
        $creator = $this->service('XFA\RMMarketplace:Purchase\CreateWithInfo', $resource, $transaction);

        $creator->setQuantity($inputData['quantity']);
        $creator->setPrice($price);

        $creator->setName($inputData['first_name'], $inputData['last_name']);
        $creator->setPhone($inputData['phone']);
        $creator->setAddress($inputData['street'], $inputData['street2'], $inputData['zip'], $inputData['city'], $inputData['state'], $inputData['country']);
        $creator->setAdditionalInfo($inputData['additional_info']);

        if ($coupon)
        {
            $creator->setCoupon($coupon);
        }

        if ($resource->xfa_rmmp_type == 'digital' && $resource->isPurchasableForFriend())
        {
            $username = $this->filter('for_username', 'str');

            /** @var \XF\Repository\User $userRepo */
            $userRepo = $this->repository('XF:User');
            $user = $userRepo->getUserByNameOrEmail($username);

            if ($user)
            {
                $creator->setForUser($user);
            }
            else if (!$resource->isPurchasable())
            {
	            // Resource must be purchased for a friend if not purchasable for oneself
	            throw $this->exception($this->noPermission());	            
            }
        }

        return $creator;
    }

    protected function setupPurchaseInfoCreate(
        \XFA\RMMarketplace\Entity\Purchase $purchase,
        $name, $phone, $address, $additionalInfo
    )
    {
        /** @var \XFA\RMMarketplace\Service\Purchase\CreateInfo $creator */
        $creator = $this->service('XFA\RMMarketplace:Purchase\CreateInfo', $purchase);

        $creator->setName($name);
        $creator->setPhone($phone);
        $creator->setAddress($address);
        $creator->setAdditionalInfo($additionalInfo);

        return $creator;
    }

    protected function setupPurchaseConversationCreate(
        \XFRM\Entity\ResourceItem $resource,
        \XFA\RMMarketplace\Entity\Purchase $purchase,
        $withInvoice
    )
    {
        $visitor = \XF::visitor();

        $title      = \XF::phrase('xfa_rmmp_your_purchase_of_x', ['resource' => $resource->title])->render();

        if ($withInvoice)
        {
            $message = \XF::phrase('xfa_rmmp_your_purchase_of_x_msg_with_invoice_link', [
                'username'      => $visitor->username,
                'resource'      => $resource->title,
                'resourceLink'  => $this->buildLink('canonical:resources', $resource),
                'invoiceLink'   => $this->buildLink('canonical:resources/market-place-purchase/invoice', $purchase),
                'sellername'    => $resource->username
            ]);
        }
        else
        {
            $message = \XF::phrase('xfa_rmmp_your_purchase_of_x_msg', [
                'username'      => $visitor->username,
                'resource'      => $resource->title,
                'resourceLink'  => $this->buildLink('canonical:resources', $resource),
                'sellername'    => $resource->username
            ]);
        }

        /** @var \XF\Service\Conversation\Creator $creator */
        $creator = $this->service('XF:Conversation\Creator', $resource->User);
        $creator->setOptions([
            'open_invite'       => false,
            'conversation_open' => true
        ]);
        $creator->setRecipientsTrusted($visitor);
        $creator->setContent($title, $message);

        return $creator;
    }

    protected function setupPurchaseTransactionEdit(\XFA\RMMarketplace\Entity\Transaction $transaction, $paymentInfo)
    {
        /** @var \XFA\RMMarketplace\Service\Transaction\Edit $editor */
        $editor = $this->service('XFA\RMMarketplace:Transaction\Edit', $transaction);

        $editor->setStatus('completed');
        $editor->setTxnId($paymentInfo['transaction_id']);
        $editor->setFeesAmount($paymentInfo['fees_amount']);

        if (isset($paymentInfo['pos_transaction_id']) && $paymentInfo['pos_transaction_id'])
        {
            $editor->setPercentageOnSaleData($paymentInfo['pos_transaction_id'], $paymentInfo['pos_amount']);
        }

        return $editor;
    }

    protected function setupShipmentStatusEdit(\XFA\RMMarketplace\Entity\Purchase $purchase, $shipmentInfo)
    {
        /** @var \XFA\RMMarketplace\Service\Purchase\EditShipmentStatus $editor */
        $editor = $this->service('XFA\RMMarketplace:Purchase\EditShipmentStatus', $purchase);

        $editor->setShipmentStatus($shipmentInfo['shipment_status'], $shipmentInfo['shipment_data']);

        return $editor;
    }
}