<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2018
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Pub\Controller;

use XF\Mvc\ParameterBag;
use XF\Pub\Controller\AbstractController;

class User extends AbstractController
{
    protected function preDispatchController($action, ParameterBag $params)
    {
        $this->assertRegistrationRequired();

        /** @var \XFRM\XF\Entity\User $visitor */
        $visitor = \XF::visitor();

        if (!$visitor->canViewResources($error))
        {
            throw $this->exception($this->noPermission($error));
        }
    }

    public function actionPurchases()
    {
        /** @var \XFA\RMMarketplace\ControllerPlugin\Purchase $purchasePlugin */
        $purchasePlugin = $this->plugin('XFA\RMMarketplace:Purchase');

        $viewParams = $purchasePlugin->getListData('buyer');

        $this->assertValidPage($viewParams['page'], $viewParams['perPage'], $viewParams['total'], 'resources/market-place-user/purchases');
        $this->assertCanonicalUrl($this->buildLink('resources/market-place-user/purchases', null, ['page' => $viewParams['page']]));

        return $this->view('', 'xfa_rmmp_user_purchases', $viewParams);
    }

    public function actionPurchasesFilters()
    {
        /** @var \XFA\RMMarketplace\ControllerPlugin\Purchase $purchasePlugin */
        $purchasePlugin = $this->plugin('XFA\RMMarketplace:Purchase');

        return $purchasePlugin->actionFilters('resources/market-place-user/purchases', false);
    }

    public function actionLicenses()
    {
        $inProfile = $this->filter('inprofile', 'uint');

        /** @var \XFA\RMMarketplace\ControllerPlugin\License $licensePlugin */
        $licensePlugin = $this->plugin('XFA\RMMarketplace:License');

        if ($inProfile)
        {
            if (\XF::visitor()->canManageAllLicenses() && $this->request()->getCookie('xfa_rmmp_dashboard_conf') == 'all')
            {
                $viewParams = $licensePlugin->getListData('all');
            }
            else
            {
                $viewParams = $licensePlugin->getListData('seller');
            }

            $viewParams['filters']['inprofile'] = 1;
        }
        else
        {
            $viewParams = $licensePlugin->getListData('buyer');
        }

        $this->assertValidPage($viewParams['page'], $viewParams['perPage'], $viewParams['total'], 'resources/market-place-user/licenses');
        $this->assertCanonicalUrl($this->buildLink('resources/market-place-user/licenses', null, ['page' => $viewParams['page']]));

        return $this->view('', 'xfa_rmmp_user_licenses', $viewParams);
    }

    public function actionLicensesFilters()
    {
        /** @var \XFA\RMMarketplace\ControllerPlugin\License $licensePlugin */
        $licensePlugin = $this->plugin('XFA\RMMarketplace:License');

        return $licensePlugin->actionFilters('all','resources/market-place-user/licenses', false);
    }
    
    public function actionInvoices()
    {	    
        // Check we have xF payment method activated
        if (\XF::options()->xfa_rmmp_mode != 'xf')
        {
            throw $this->exception($this->noPermission());
        }
        	    
        /** @var \XFRM\XF\Entity\User $visitor */
        $visitor = \XF::visitor();

        if (!$visitor->isASeller() || \XF::options()->xfa_rmmp_mode != 'xf')
        {
            throw $this->exception($this->noPermission());
        }
        	
        // Get unpaid invoices
		$invoicesFinder = \XF::finder('XFA\RMMarketplace:MonthlyInvoice');
		$invoicesFinder->where('status', '=', 'pending');
		$invoicesFinder->where('user_id', '=', $visitor->user_id);
		$invoicesFinder->order('invoice_date', 'asc');
		
		$unpaidInvoices = $invoicesFinder->fetch();
		
		// Get other invoices
		$invoicesFinder = \XF::finder('XFA\RMMarketplace:MonthlyInvoice');
		$invoicesFinder->where('status', '!=', 'pending');
		$invoicesFinder->where('user_id', $visitor->user_id);
		$invoicesFinder->order('invoice_date', 'desc');
		
        $total      = $invoicesFinder->total();
        $page       = $this->filterPage();
        $perPage	= $this->options()->xfa_rmmp_purchasePerPage;

        $this->assertValidPage($page, $perPage, $total, 'resources/market-place-user/invoices');
        $this->assertCanonicalUrl($this->buildLink('resources/market-place-user/invoices', null, ['page' => $page]));

        $invoices = $invoicesFinder->limitByPage($page, $perPage)->fetch();

		return $this->view('', 'xfa_rmmp_user_invoices', ['unpaidInvoices' => $unpaidInvoices, 'invoices' => $invoices]);
    }
    
    public function actionInvoiceDetails()
    {
        // Check we have xF payment method activated
        if (\XF::options()->xfa_rmmp_mode != 'xf')
        {
            throw $this->exception($this->noPermission());
        }	    
	    
        /** @var \XFRM\XF\Entity\User $visitor */
        $visitor = \XF::visitor();

        if (!$visitor->isASeller() || \XF::options()->xfa_rmmp_mode != 'xf')
        {
            throw $this->exception($this->noPermission());
        }	    
	    
	    // Get invoice or error (also if not user invoice)
	    $invoiceId 	= $this->filter('invoice_id', 'uint');
	    $invoice 	= $this->finder('XFA\RMMarketplace:MonthlyInvoice')->whereId($invoiceId)->fetchOne();
	    if (!$invoice || $invoice->user_id != $visitor->user_id)
	    {
	        throw $this->exception($this->noPermission());	    
	    }
	    
	    // Get all transactions from this invoice
	    $transactions = $this->finder('XFA\RMMarketplace:Transaction')->where('pos_invoice_id', '=', $invoice->invoice_id)->with(['Purchase', 'Purchase.Resource'])->order('Purchase.purchase_date', 'desc');
	    if (!$transactions->total())
	    {
	        throw $this->exception($this->noPermission());	
	    }
	    
	    return $this->view('', 'xfa_rmmp_user_invoice_details', ['transactions' => $transactions]);
    }
    
    public function actionInvoicePay()
    {
        // Check we have xF payment method activated
        if (\XF::options()->xfa_rmmp_mode != 'xf')
        {
            throw $this->exception($this->noPermission());
        }
	    
        /** @var \XFRM\XF\Entity\User $visitor */
        $visitor = \XF::visitor();

        if (!$visitor->isASeller() || \XF::options()->xfa_rmmp_mode != 'xf')
        {
            throw $this->exception($this->noPermission());
        }	    
	    
	    // Get invoice or error (also if not user invoice)
	    $invoiceId 	= $this->filter('invoice_id', 'uint');
	    $invoice 	= $this->finder('XFA\RMMarketplace:MonthlyInvoice')->whereId($invoiceId)->fetchOne();
	    if (!$invoice || $invoice->user_id != $visitor->user_id)
	    {
	        throw $this->exception($this->noPermission());	    
	    }
	    
        // Check purchasability
        $purchasable = $this->em()->find('XF:Purchasable', 'xfa_rmmp_invoice', 'AddOn');
        if (!$purchasable || !$purchasable->isActive())
        {
            return $this->message(\XF::phrase('xfa_rmmp_no_purchase_cant_be_done_right_now'));
        }

        // Get payment profiles
		$profiles = $this->finder('XF:PaymentProfile')
							->where('payment_profile_id', \XF::options()->xfa_rmmp_paymentProfileIds)
							->where('active', 1)
							->fetch();
        
        if (!count($profiles))
        {
            throw $this->exception($this->noPermission());
        }
                
        $viewParams = [
            'invoice'  	=> $invoice,
            'profiles'	=> $profiles
        ];
        return $this->view('', 'xfa_rmmp_user_invoice_pay', $viewParams); 	    
    }
}