<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2021
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Purchasable;

use XF\Payment\CallbackState;
use XF\Purchasable\AbstractPurchasable;
use XF\Purchasable\Purchase;

class MarketplaceFeature extends AbstractPurchasable
{
    public function getTitle()
    {
        return \XF::phrase('xfa_rmmp_purchase_resource_featuring');
    }

    public function getPurchaseFromRequest(\XF\Http\Request $request, \XF\Entity\User $purchaser, &$error = null)
    {
        $profileId = $request->filter('payment_profile_id', 'uint');
        
        // Get featuring duration
        /** @var \XFA\RMMarketplace\Entity\FeaturingDuration $duration */
        $duration = \XF::em()->find('XFA\RMMarketplace:FeaturingDuration', $request->filter('duration_id', 'uint'));
        if (!$duration)
        {

            $error = \XF::phrase('xfa_rmmp_requested_featuring_duration_not_found');
            return false;	        
        }
        
        // Get resource
        /** @var \XFRM\Entity\ResourceItem $resource */
        $resource = \XF::em()->find('XFRM:ResourceItem', $request->filter('resource_id', 'uint'));
        if (!$resource)
        {

            $error = \XF::phrase('xfa_rmmp_resource_not_found');
            return false;	        
        }        
        
        // Get payment profile
        $paymentProfile = \XF::app()->finder('XF:PaymentProfile')
							->where('payment_profile_id', $profileId)
							->where('active', 1)
							->fetchOne();


        if (!$paymentProfile || !$paymentProfile->active)
        {
            $error = \XF::phrase('please_choose_valid_payment_profile_to_continue_with_your_purchase');
            return false;
        }

        // Create an empty transaction
        /** @var \XFA\RMMarketplace\Service\Transaction\Create $creator */
        $creator = \XF::service('XFA\RMMarketplace:Transaction\Create', $resource);
        $transaction = $creator->save();

        // Create the purchase
        /** @var \XFA\RMMarketplace\Service\FeaturingPurchase\Create $creator */
        $creator = \XF::service('XFA\RMMarketplace:FeaturingPurchase\Create', $resource, $transaction);
        $creator->setPrice($duration->amount);
        $creator->setDuration($duration);
        $purchase = $creator->save();

        return $this->getPurchaseObject($paymentProfile, $purchase, $purchaser);
    }

    public function getPurchasableFromExtraData(array $extraData)
    {
        $output = [
            'title' 		=> '',
            'purchasable' 	=> null
        ];

        $purchase = \XF::em()->find('XFA\RMMarketplace:FeaturingPurchase', $extraData['purchase_id']);
        if ($purchase)
        {
            $output['title'] 		= \XF::phrase('xfa_rmmp_purchase_resource_featuring') . ' #' . $purchase->purchase_id;
            $output['purchasable'] 	= $purchase;
        }
        return $output;
    }

    public function getPurchaseFromExtraData(array $extraData, \XF\Entity\PaymentProfile $paymentProfile, \XF\Entity\User $purchaser, &$error = null)
    {
        $purchase = $this->getPurchasableFromExtraData($extraData);
        if (!$purchase['purchasable'])
        {
            $error = \XF::phrase('this_item_cannot_be_purchased_at_moment');
            return false;
        }

        if (!$paymentProfile)
        {
            $error = \XF::phrase('selected_payment_profile_is_not_valid_for_this_purchase');
            return false;
        }

        return $this->getPurchaseObject($paymentProfile, $purchase['purchasable'], $purchaser);
    }

    public function getPurchaseObject(\XF\Entity\PaymentProfile $paymentProfile, $purchasable, \XF\Entity\User $purchaser)
    {
        $purchase = new Purchase();

        $purchase->description          = $purchasable->Resource->title .' - ' . $purchasable->duration;
        $purchase->title                = \XF::phrase('xfa_rmmp_purchase_resource_featuring')->render() . ' #' . $purchasable->purchase_id . ': ' . $purchase->description;        
        $purchase->cost                 = $purchasable->price;
        $purchase->currency             = \XF::options()->xfa_rmmp_featuring_purchase_currency;
        $purchase->recurring            = false;
        $purchase->purchaser            = $purchaser;
        $purchase->paymentProfile       = $paymentProfile;
        $purchase->purchasableTypeId    = $this->purchasableTypeId;
        $purchase->purchasableId        = $purchasable->purchase_id;
        $purchase->purchasableTitle     = \XF::phrase('xfa_rmmp_purchase_resource_featuring')->render() . ' #' . $purchasable->purchase_id;
        $purchase->extraData = [
            'purchase_id' => $purchasable->purchase_id
        ];

        $router = \XF::app()->router('public');

		$purchase->returnUrl = $router->buildLink('canonical:resources', $purchasable->Resource);

        $purchase->cancelUrl = $router->buildLink('canonical:resources', $purchasable->Resource);

        return $purchase;
    }

    public function completePurchase(CallbackState $state)
    {
        if ($state->legacy)
        {
            $purchaseRequest 	= null;
            $purchaseId 		= $state->purchase->purchase_id;
        }
        else
        {
            $purchaseRequest 	= $state->getPurchaseRequest();
            $purchaseId 		= $purchaseRequest->extra_data['purchase_id'];
        }
        
		$paymentResult = $state->paymentResult;

		$purchaseFinder = \XF::finder('XFA\RMMarketplace:FeaturingPurchase');
		$purchaseFinder->with(['Resource', 'Transaction']);
		$purchaseFinder->whereId($purchaseId);
		$purchase = $purchaseFinder->fetchOne();
		
		$resource = $purchase->Resource;

        switch ($paymentResult)
        {
            case CallbackState::PAYMENT_RECEIVED:
            case CallbackState::PAYMENT_REINSTATED:
				// Set transaction as completed
				$purchase->Transaction->fastUpdate('status', 'completed');				
            
				// Validate the purchase
                $validator = \XF::service('XFA\RMMarketplace:FeaturingPurchase\Validate', $purchase);
                $validator->validate();				
				
                $state->logType = 'payment';
                
                if ($paymentResult == CallbackState::PAYMENT_RECEIVED)
                {                
                	$state->logMessage = 'Marketplace featuring payment received, purchase validated.';
				}
				else
				{
                    $state->logMessage = 'Marketplace featuring reversal cancelled, purchase revalidated.';
				}
				
                break;
        }
    }

    public function reversePurchase(CallbackState $state)
    {
        if ($state->legacy)
        {
            $purchaseRequest 	= null;
            $purchaseId 		= $state->purchase->purchase_id;
        }
        else
        {
            $purchaseRequest 	= $state->getPurchaseRequest();
            $purchaseId			= $purchaseRequest->extra_data['purchase_id'];
        }

		$purchaseFinder = \XF::finder('XFA\RMMarketplace:FeaturingPurchase');
		$purchaseFinder->with(['Resource', 'Transaction']);
		$purchaseFinder->whereId($purchaseId);
		$purchase = $purchaseFinder->fetchOne();

        /** @var \XFA\RMMarketplace\Service\FeaturingPurchase\Refund $refunder */
        \XF::service('XFA\RMMarketplace:FeaturingPurchase\Refund', $purchase)->refund();

        $state->logType = 'cancel';
        $state->logMessage = 'Marketplace featuring payment refunded/reversed, downgraded.';
    }

    public function getPurchasablesByProfileId($profileId)
    {
        return [];
    }
    
    public function sendPaymentReceipt(CallbackState $state)
	{
		return;
	}
}