<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2021
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Purchasable;

use XF\Payment\CallbackState;
use XF\Purchasable\AbstractPurchasable;
use XF\Purchasable\Purchase;

class MarketplaceInvoice extends AbstractPurchasable
{
    public function getTitle()
    {
        return \XF::phrase('xfa_rmmp_monthly_invoice');
    }

    public function getPurchaseFromRequest(\XF\Http\Request $request, \XF\Entity\User $purchaser, &$error = null)
    {
        $profileId = $request->filter('payment_profile_id', 'uint');
        
        // Get invoice
        $invoiceId = $request->filter('invoice_id', 'uint');
        
        /** @var \XFA\RMMarketplace\Entity\MonthlyInvoice $invoice */
        $invoice = \XF::em()->find('XFA\RMMarketplace:MonthlyInvoice', $invoiceId);
        if (!$invoice || $invoice->user_id != \XF::visitor()->user_id)
        {
            $error = \XF::phrase('xfa_rmmp_invoice_not_found');
            return false;
        }
        

        // Get payment profile
        $paymentProfile = \XF::app()->finder('XF:PaymentProfile')
							->where('payment_profile_id', $profileId)
							->where('active', 1)
							->fetchOne();

        if (!$paymentProfile || !$paymentProfile->active)
        {
            $error = \XF::phrase('please_choose_valid_payment_profile_to_continue_with_your_purchase');
            return false;
        }

        return $this->getPurchaseObject($paymentProfile, $invoice, $purchaser);
    }

    public function getPurchasableFromExtraData(array $extraData)
    {
        $output = [
            'title' 		=> '',
            'purchasable' 	=> null
        ];

        $invoice = \XF::em()->find('XFA\RMMarketplace:MonthlyInvoice', $extraData['invoice_id']);
        if ($invoice)
        {
            $output['title'] 		= \XF::phrase('xfa_rmmp_invoice') . ' #' . $invoice->invoice_id;
            $output['purchasable'] 	= $purchase;
        }
        return $output;
    }

    public function getPurchaseFromExtraData(array $extraData, \XF\Entity\PaymentProfile $paymentProfile, \XF\Entity\User $purchaser, &$error = null)
    {
        $purchase = $this->getPurchasableFromExtraData($extraData);
        if (!$purchase['purchasable'])
        {
            $error = \XF::phrase('this_item_cannot_be_purchased_at_moment');
            return false;
        }

        if (!$paymentProfile)
        {
            $error = \XF::phrase('selected_payment_profile_is_not_valid_for_this_purchase');
            return false;
        }

        return $this->getPurchaseObject($paymentProfile, $purchase['purchasable'], $purchaser);
    }

    public function getPurchaseObject(\XF\Entity\PaymentProfile $paymentProfile, $purchasable, \XF\Entity\User $purchaser)
    {
        $purchase = new Purchase();

        $purchase->description          = \XF::phrase('xfa_rmmp_invoice')->render() . ' #' . $purchasable->invoice_id;
        $purchase->title                = \XF::phrase('xfa_rmmp_invoice')->render() . ' #' . $purchasable->invoice_id;
        $purchase->cost                 = $purchasable->price;
        $purchase->currency             = $purchasable->currency;
        $purchase->recurring            = false;
        $purchase->purchaser            = $purchaser;
        $purchase->paymentProfile       = $paymentProfile;
        $purchase->purchasableTypeId    = $this->purchasableTypeId;
        $purchase->purchasableId        = $purchasable->invoice_id;
        $purchase->purchasableTitle     = \XF::phrase('xfa_rmmp_invoice')->render() . ' #' . $purchasable->invoice_id;
        $purchase->extraData = [
            'invoice_id' => $purchasable->invoice_id
        ];

        $router = \XF::app()->router('public');

		$purchase->returnUrl = $router->buildLink('canonical:resources/market-place-user/invoices');

        $purchase->cancelUrl = $router->buildLink('canonical:resources/market-place-user/invoices');

        return $purchase;
    }

    public function completePurchase(CallbackState $state)
    {
        if ($state->legacy)
        {
            $purchaseRequest 	= null;
            $invoiceId			= $state->purchase->invoice_id;
        }
        else
        {
            $purchaseRequest 	= $state->getPurchaseRequest();
            $invoiceId 			= $purchaseRequest->extra_data['invoice_id'];
        }
        
		$paymentResult = $state->paymentResult;

		$invoiceFinder = \XF::finder('XFA\RMMarketplace:MonthlyInvoice');
		$invoiceFinder->whereId($invoiceId);
		$invoice = $invoiceFinder->fetchOne();
		
        switch ($paymentResult)
        {
            case CallbackState::PAYMENT_RECEIVED:
            case CallbackState::PAYMENT_REINSTATED:
				// Validate the invoice	
				\XF::service('XFA\RMMarketplace:PosInvoice\Validate', $invoice)->validate();
				
                $state->logType = 'payment';
                
                if ($paymentResult == CallbackState::PAYMENT_RECEIVED)
                {                
                	$state->logMessage = 'Marketplace invoice payment received, purchase validated.';
				}
				else
				{
                    $state->logMessage = 'Marketplace invoice reversal cancelled, purchase revalidated.';
				}
				
                break;
        }
    }

    public function reversePurchase(CallbackState $state)
    {
        if ($state->legacy)
        {
            $purchaseRequest 	= null;
            $invoiceId			= $state->purchase->invoice_id;
        }
        else
        {
            $purchaseRequest 	= $state->getPurchaseRequest();
            $invoiceId 			= $purchaseRequest->extra_data['invoice_id'];
        }

		$invoiceFinder = \XF::finder('XFA\RMMarketplace:MonthlyInvoice');
		$invoiceFinder->whereId($invoiceId);
		$invoice = $invoiceFinder->fetchOne();		
		
		// Refund the invoice
        \XF::service('XFA\RMMarketplace:PosInvoice\Refund', $invoice)->refund();		

        $state->logType = 'cancel';
        $state->logMessage = 'Marketplace invoice payment refunded/reversed, downgraded.';
    }

    public function getPurchasablesByProfileId($profileId)
    {
        return [];
    }
    
    public function sendPaymentReceipt(CallbackState $state)
	{
		return;
	}
}