<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2021
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Purchasable;

use XF\Payment\CallbackState;
use XF\Purchasable\AbstractPurchasable;
use XF\Purchasable\Purchase;

class MarketplacePurchase extends AbstractPurchasable
{
    public function getTitle()
    {
        return \XF::phrase('xfa_rmmp_purchase');
    }

    public function getPurchaseFromRequest(\XF\Http\Request $request, \XF\Entity\User $purchaser, &$error = null)
    {
        $profileId = $request->filter('payment_profile_id', 'uint');

        // Get resource
        $resourceId = $request->filter('resource_id', 'uint');
        $resource = \XF::em()->find('XFRM:ResourceItem', $resourceId);

        if (!$resource || $resource->resource_state != 'visible')
        {
            $error = \XF::phrase('this_item_cannot_be_purchased_at_moment');
            return false;
        }

        // Get coupon if any
        $couponCode = $request->filter('coupon_code', 'str');
        $coupon     = \XF::finder('XFA\RMMarketplace:Coupon')
            ->where('code', $couponCode)
            ->fetchOne();

        // Check coupon is usable on resource
        $usableCoupon = $coupon ? $coupon->isUsableOnResourceByUser('purchase', $resource, \XF::visitor()) : false;
        if ($couponCode && !$usableCoupon)
        {
            $error = \XF::phrase('xfa_rmmp_invalid_coupon');
            return false;
        }
        
        // For physical resources check additional info is set
        if ($resource->xfa_rmmp_type == 'physical')
        {
            $additionalData = $request->filter([
                'first_name'        => 'str',
                'last_name'         => 'str',
                'phone'             => 'str',
                'street'            => 'str',
                'street2'           => 'str',
                'city'              => 'str',
                'zip'               => 'str',
                'state'             => 'str',
                'country'           => 'str',
                'additional_info'   => 'str'
            ]);

            /* Check info fields */
            if (empty($additionalData['first_name'])
                || empty($additionalData['last_name'])
                || empty($additionalData['street'])
                || empty($additionalData['city'])
                || empty($additionalData['zip'])
                || empty($additionalData['country']))
            {
                $error = \XF::phrase('xfa_rmmp_must_set_info_fields');
                return false;
            }	        
        }

        // Get payment profile
        /** @var \XF\Entity\PaymentProfile $paymentProfile */
        switch($resource->Category->xfa_rmmp_conf)
        {
            case 1: // User payment data
				$paymentProfile = \XF::app()->finder('XF:PaymentProfile')
									->where('payment_profile_id', $profileId)
									->where('xfa_rmmp_user_id', $resource->user_id)
									->where('active', 1)
									->fetchOne();						
                break;
            case 2:
            case 3:
            default:
                $paymentProfile = \XF::app()->finder('XF:PaymentProfile')
									->where('payment_profile_id', $profileId)
									->where('active', 1)
									->fetchOne();
                break;
        }
        
        // Get for User data if any
        $forUsername = $request->filter('for_username', 'str');

        /** @var \XF\Repository\User $userRepo */
        $userRepo = \XF::repository('XF:User');
        $forUser = $userRepo->getUserByNameOrEmail($forUsername);	        

        if (!$paymentProfile || !$paymentProfile->active)
        {
            $error = \XF::phrase('please_choose_valid_payment_profile_to_continue_with_your_purchase');
            return false;
        }

        if (!$resource->isPurchasable())
        {
			if (!$resource->isPurchasableForFriend() || !$forUser)
			{
	            $error = \XF::phrase('this_item_cannot_be_purchased_at_moment_or_already_own_can_only_purchase_friend');
	            return false;
			}    
        }

        // Check CGV were accepted if any
        if ($resource->isRequiringCGVValidation() && !$request->filter('xfa_agree', 'bool'))
        {
            $error = \XF::phrase('xfa_rmmp_must_accept_cgv');
            return false;
        }

        // Check quantity if needed
        $quantity = $request->filter('quantity', 'uint', 1);
        if ($resource->canBePurchasedInMultipleQty())
        {
            $maxQty = $resource->getPurchasableQty();
            if ($maxQty > 0 && $quantity > $maxQty)
            {
                $error = \XF::phrase('xfa_rmmp_not_allowed_purchase_that_quantity');
                return false;
            }
        }

        // Get actual price
        $price = $resource->getComputedResourcePrice($quantity, $coupon);

        // Check if percentage on sales for category or globally
        if ($resource->bypassPercentageOnSale())
        {
            $withPercentageOnSale	= 0;
            $percentageOnSale       = 0;
        }
        else
        {
	        if ($resource->Category->xfa_rmmp_sale_percentage > 0.00)
	        {
	            $withPercentageOnSale       = 1;
	            $percentageOnSale           = $resource->Category->xfa_rmmp_sale_percentage;
	        }
	        else if (\XF::options()->xfa_rmmp_sale_percentage > 0.00)
	        {
	            $withPercentageOnSale       = 1;
	            $percentageOnSale           = \XF::options()->xfa_rmmp_sale_percentage;
	        }
	        else
	        {
	            $withPercentageOnSale       = 0;
	            $percentageOnSale           = 0;
	        }
		}

        // Compute amounts based on that
        if ($withPercentageOnSale)
        {
            $adminAmount    = round(($price * $percentageOnSale / 100), 2);
        }
        else
        {
            $adminAmount    = 0;
        }
        
        // Compute shipping if any
        if ($resource->xfa_rmmp_type == 'physical' && $resource->PhysicalProduct->shipping)
        {
	        $shipping = round($request->filter('shipping', 'float'), 2);
        }
        else
        {
	        $shipping = 0;
        }

        // Update payment profile address requirement option
        $options = $paymentProfile->options;
        if (isset($options['require_address']))
        {
            if ($resource->xfa_rmmp_type == 'physical')
            {
                $options['require_address'] = 1;
            }
            else
            {
                $options['require_address'] = 0;
            }

            $paymentProfile->options = $options;
        }

        // Create an empty transaction
        /** @var \XFA\RMMarketplace\Service\Transaction\Create $creator */
        $creator = \XF::service('XFA\RMMarketplace:Transaction\Create', $resource);
        $creator->setPercentageOnSaleData('', $adminAmount, true);
        $creator->setShipping($shipping);
        $transaction = $creator->save();

        // Create the purchase
        /** @var \XFA\RMMarketplace\Service\Purchase\Create $creator */
        $creator = \XF::service('XFA\RMMarketplace:Purchase\Create', $resource, $transaction);
        $creator->setQuantity($quantity);
        $creator->setPrice($price + $shipping);
        
        if ($resource->xfa_rmmp_type == 'digital' && $resource->isPurchasableForFriend() && $forUser)
        {
	        $creator->setForUser($forUser);
        }

        if ($coupon)
        {
            $creator->setCoupon($coupon);
        }
        
        $purchase = $creator->save();

        // Create inactive license(s) if digital product
        if ($resource->xfa_rmmp_type == 'digital')
        {
            for ($i = 0; $i < $quantity; $i++)
            {
                /** @var \XFA\RMMarketplace\Service\License\Create $creator */
                $creator = \XF::service('XFA\RMMarketplace:License\Create', $resource);
                $creator->setPurchaseId($purchase->purchase_id);

                if ($resource->DigitalProduct->duration)
                {
                    $creator->setEndDateWithDuration($resource->DigitalProduct->getDurationString());
                }
                
		        if ($purchase->for_user_id != 0)
		        {
		            $creator->setOwner($purchase->ForUser);
		        }        

                $license = $creator->save();
            }
        }
        else
        {
            $license = null;
        }
        
        // Add additional data if physical product
        if ($resource->xfa_rmmp_type == 'physical')
		{
	        /** @var \XFA\RMMarketplace\Service\Purchase\CreateInfo $creator */
	        $creator = \XF::service('XFA\RMMarketplace:Purchase\CreateInfo', $purchase);
			
			$name = $additionalData['last_name'] . ' ' . $additionalData['first_name'];
			
			$address = $additionalData['street'];

			if (!empty($additionalData['street2']))
			{
				$address .= "\r\n" . $additionalData['street2'];
			}
			
			$address .= "\r\n" . $additionalData['zip'] . ' ' . $additionalData['city'];
			
			if (!empty($additionalData['state']))
			{
				$address .= "\r\n" . $additionalData['state'];
			}
			
			$address .= "\r\n" . $additionalData['country'];
	
	        $creator->setName($name);
	        $creator->setPhone($additionalData['phone']);
	        $creator->setAddress($address);
	        $creator->setAdditionalInfo($additionalData['additional_info']);			
	        
	        $creator->save();
		}

        return $this->getPurchaseObject($paymentProfile, $purchase, $purchaser);
    }

    public function getPurchasableFromExtraData(array $extraData)
    {
        $output = [
            'title' 		=> '',
            'purchasable' 	=> null
        ];

        $purchase = \XF::em()->find('XFA\RMMarketplace:Purchase', $extraData['purchase_id']);
        if ($purchase)
        {
            $output['title'] 		= \XF::phrase('xfa_rmmp_purchase') . ' - ' . $purchase->purchase_id;
            $output['purchasable'] 	= $purchase;
        }
        return $output;
    }

    public function getPurchaseFromExtraData(array $extraData, \XF\Entity\PaymentProfile $paymentProfile, \XF\Entity\User $purchaser, &$error = null)
    {
        $purchase = $this->getPurchasableFromExtraData($extraData);
        if (!$purchase['purchasable'])
        {
            $error = \XF::phrase('this_item_cannot_be_purchased_at_moment');
            return false;
        }

        if (!$paymentProfile)
        {
            $error = \XF::phrase('selected_payment_profile_is_not_valid_for_this_purchase');
            return false;
        }

        return $this->getPurchaseObject($paymentProfile, $purchase['purchasable'], $purchaser);
    }

    public function getPurchaseObject(\XF\Entity\PaymentProfile $paymentProfile, $purchasable, \XF\Entity\User $purchaser)
    {
        $purchase = new Purchase();

        $purchase->description          = $purchasable->Resource->title . (($purchasable->quantity > 1) ? ' - ' . \XF::phrase('xfa_rmmp_qty')->render() . ' - ' . $purchasable->quantity . '' : '');
        $purchase->title                = \XF::phrase('xfa_rmmp_purchase')->render() . ' - ' . $purchasable->purchase_id . ': ' . $purchase->description;;
        $purchase->cost                 = $purchasable->price;
        $purchase->currency             = $purchasable->currency;
        $purchase->recurring            = false;
        $purchase->purchaser            = $purchaser;
        $purchase->paymentProfile       = $paymentProfile;
        $purchase->purchasableTypeId    = $this->purchasableTypeId;
        $purchase->purchasableId        = $purchasable->purchase_id;
        $purchase->purchasableTitle     = \XF::phrase('xfa_rmmp_purchase')->render() . ' - ' . $purchasable->purchase_id;
        $purchase->extraData = [
            'purchase_id' => $purchasable->purchase_id
        ];

        $router = \XF::app()->router('public');

        if ($purchasable->Resource->xfa_rmmp_type == 'digital' && !$purchasable->for_user_id)
        {
            $purchase->returnUrl = $router->buildLink('canonical:resources/market-place-user/licenses');
        }
        else
        {
            $purchase->returnUrl = $router->buildLink('canonical:resources/market-place-user/purchases');
        }

        $purchase->cancelUrl = $router->buildLink('canonical:resources', $purchasable->Resource);

        return $purchase;
    }

    public function completePurchase(CallbackState $state)
    {
        if ($state->legacy)
        {
            $purchaseRequest 	= null;
            $purchaseId 		= $state->purchase->purchase_id;
        }
        else
        {
            $purchaseRequest 	= $state->getPurchaseRequest();
            $purchaseId 		= $purchaseRequest->extra_data['purchase_id'];
        }
        
		$paymentResult = $state->paymentResult;

		$purchaseFinder = \XF::finder('XFA\RMMarketplace:Purchase');
		$purchaseFinder->with(['Resource', 'Transaction', 'License']);
		$purchaseFinder->whereId($purchaseId);
		$purchase = $purchaseFinder->fetchOne();
		
		$resource = $purchase->Resource;

        switch ($paymentResult)
        {
            case CallbackState::PAYMENT_RECEIVED:
            case CallbackState::PAYMENT_REINSTATED:
		        // If Physical product, need to decrease quantity
		        if ($resource->xfa_rmmp_type == 'physical' && $resource->PhysicalProduct->quantity)
		        {
		            $resource->PhysicalProduct->quantity--;
		            $resource->PhysicalProduct->save();
		        }				
            
				// Set transaction as completed
				$purchase->Transaction->fastUpdate('status', 'completed');				
            
				// Validate the purchase
                $validator = \XF::service('XFA\RMMarketplace:Purchase\Validate', $purchase);
                $validator->validate();

                $state->logType = 'payment';
                
                if ($paymentResult == CallbackState::PAYMENT_RECEIVED)
                {                
                	$state->logMessage = 'Marketplace payment received, purchase validated.';
				}
				else
				{
                    $state->logMessage = 'Marketplace reversal cancelled, purchase revalidated.';
				}
				
                break;
        }
    }

    public function reversePurchase(CallbackState $state)
    {
        if ($state->legacy)
        {
            $purchaseRequest 	= null;
            $purchaseId 		= $state->purchase->purchase_id;
        }
        else
        {
            $purchaseRequest 	= $state->getPurchaseRequest();
            $purchaseId			= $purchaseRequest->extra_data['purchase_id'];
        }

		$purchaseFinder = \XF::finder('XFA\RMMarketplace:Purchase');
		$purchaseFinder->with(['Resource', 'Transaction', 'License']);
		$purchaseFinder->whereId($purchaseId);
		$purchase = $purchaseFinder->fetchOne();

        /** @var \XFA\RMMarketplace\Service\Purchase\Refund $refunder */
        \XF::service('XFA\RMMarketplace:Purchase\Refund', $purchase)->refund();

        $state->logType = 'cancel';
        $state->logMessage = 'Marketplace payment refunded/reversed, downgraded.';
    }

    public function getPurchasablesByProfileId($profileId)
    {
        return [];
    }
    
    public function sendPaymentReceipt(CallbackState $state)
	{
		return;
	}
}