<?php
/*************************************************************************
 * XenForo RM Marketplace - Xen Factory (c) 2015-2021
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Purchasable;

use XF\Payment\CallbackState;
use XF\Purchasable\AbstractPurchasable;
use XF\Purchasable\Purchase;

class MarketplaceRenew extends AbstractPurchasable
{
    public function getTitle()
    {
        return \XF::phrase('xfa_rmmp_renewal');
    }

    public function getPurchaseFromRequest(\XF\Http\Request $request, \XF\Entity\User $purchaser, &$error = null)
    {
        $profileId = $request->filter('payment_profile_id', 'uint');
        
        // Get license and check it's renewable
        $licenseId = $request->filter('license_id', 'uint');
        
        $extraWith= ['Resource', 'Resource.DigitalProduct'];

        /** @var \XFA\RMMarketplace\Entity\License $license */
        $license = \XF::em()->find('XFA\RMMarketplace:License', $licenseId, $extraWith);
        if (!$license || !$license->isOwner())
        {
            $error = \XF::phrase('xfa_rmmp_license_not_found');
            return false;
        }
        
        if (!$license->Resource->DigitalProduct->duration || !$license->canBeRenewed())
        {
            $error = \XF::phrase('xfa_rmmp_license_cant_be_renewed');
            return false;
        }
        
        $resource = $license->Resource;

        // Get coupon if any
        $couponCode = $request->filter('coupon_code', 'str');
        $coupon     = \XF::finder('XFA\RMMarketplace:Coupon')
            ->where('code', $couponCode)
            ->fetchOne();

        // Check coupon is usable on resource
        $usableCoupon = $coupon ? $coupon->isUsableOnResourceByUser('renewal', $resource, \XF::visitor()) : false;
        if ($couponCode && !$usableCoupon)
        {
            $error = \XF::phrase('xfa_rmmp_invalid_coupon');
            return false;
        }

        // Get payment profile
        switch($resource->Category->xfa_rmmp_conf)
        {
            case 1: // User payment data
				$paymentProfile = \XF::app()->finder('XF:PaymentProfile')
									->where('payment_profile_id', $profileId)
									->where('xfa_rmmp_user_id', $resource->user_id)
									->where('active', 1)
									->fetchOne();						
                break;
            case 2: // Category payment data not allowed in this mode
            case 3:
            default:
                $paymentProfile = \XF::app()->finder('XF:PaymentProfile')
									->where('payment_profile_id', $profileId)
									->where('active', 1)
									->fetchOne();
                break;
        }

        if (!$paymentProfile || !$paymentProfile->active)
        {
            $error = \XF::phrase('please_choose_valid_payment_profile_to_continue_with_your_purchase');
            return false;
        }

        // Check CGV were accepted if any
        if ($resource->isRequiringCGVValidation() && !$request->filter('xfa_agree', 'bool'))
        {
            $error = \XF::phrase('xfa_rmmp_must_accept_cgv');
            return false;
        }

        // Get actual price
        $price = $resource->DigitalProduct->getComputedRenewalPrice($coupon);
        
        // Check if percentage on sales for category or globally
        if ($resource->bypassPercentageOnSale())
        {    
            $withPercentageOnSale	= 0;
            $percentageOnSale       = 0;    
	    }
	    else
	    {
	        if ($resource->Category->xfa_rmmp_sale_percentage > 0.00)
	        {
	            $withPercentageOnSale       = 1;
	            $percentageOnSale           = $resource->Category->xfa_rmmp_sale_percentage;
	        }
	        else if (\XF::options()->xfa_rmmp_sale_percentage > 0.00)
	        {
	            $withPercentageOnSale       = 1;
	            $percentageOnSale           = \XF::options()->xfa_rmmp_sale_percentage;
	        }
	        else
	        {
	            $withPercentageOnSale       = 0;
	            $percentageOnSale           = 0;
	        }
	    }

        /* Compute amounts based on that */
        if ($withPercentageOnSale)
        {
            $adminAmount    = round(($price * $percentageOnSale / 100), 2);
        }
        else
        {
            $adminAmount    = 0;
        }        

        // Create an empty transaction
        /** @var \XFA\RMMarketplace\Service\Transaction\Create $creator */
        $creator = \XF::service('XFA\RMMarketplace:Transaction\Create', $resource);
        $creator->setPercentageOnSaleData('', $adminAmount);
        $transaction = $creator->save();

        // Create the purchase
        /** @var \XFA\RMMarketplace\Service\Purchase\Create $creator */
        $creator = \XF::service('XFA\RMMarketplace:Purchase\Create', $resource, $transaction, $license);
        $creator->setType('renewal');
        $creator->setPrice($price);

        if ($coupon)
        {
            $creator->setCoupon($coupon);
        }

        $purchase = $creator->save();

        return $this->getPurchaseObject($paymentProfile, $purchase, $purchaser);
    }

    public function getPurchasableFromExtraData(array $extraData)
    {
        $output = [
            'title' 		=> '',
            'purchasable' 	=> null
        ];

        $purchase = \XF::em()->find('XFA\RMMarketplace:Purchase', $extraData['purchase_id']);
        if ($purchase)
        {
            $output['title'] 		= \XF::phrase('xfa_rmmp_purchase') . ' - ' . $purchase->purchase_id;
            $output['purchasable'] 	= $purchase;
        }
        return $output;
    }

    public function getPurchaseFromExtraData(array $extraData, \XF\Entity\PaymentProfile $paymentProfile, \XF\Entity\User $purchaser, &$error = null)
    {
        $purchase = $this->getPurchasableFromExtraData($extraData);
        if (!$purchase['purchasable'])
        {
            $error = \XF::phrase('this_item_cannot_be_purchased_at_moment');
            return false;
        }

        if (!$paymentProfile)
        {
            $error = \XF::phrase('selected_payment_profile_is_not_valid_for_this_purchase');
            return false;
        }

        return $this->getPurchaseObject($paymentProfile, $purchase['purchasable'], $purchaser);
    }

    public function getPurchaseObject(\XF\Entity\PaymentProfile $paymentProfile, $purchasable, \XF\Entity\User $purchaser)
    {
        $purchase = new Purchase();

        $purchase->description          = $purchasable->Resource->title . ' - ' . \XF::phrase('xfa_rmmp_renewal')->render();
        $purchase->title                = \XF::phrase('xfa_rmmp_renewal')->render() . ' - ' . $purchasable->purchase_id . ': ' . $purchase->description;
        $purchase->cost                 = $purchasable->price;
        $purchase->currency             = $purchasable->currency;
        $purchase->recurring            = false;
        $purchase->purchaser            = $purchaser;
        $purchase->paymentProfile       = $paymentProfile;
        $purchase->purchasableTypeId    = $this->purchasableTypeId;
        $purchase->purchasableId        = $purchasable->purchase_id;
        $purchase->purchasableTitle     = \XF::phrase('xfa_rmmp_renewal')->render() . ' - ' . $purchasable->purchase_id;
        $purchase->extraData = [
            'purchase_id' => $purchasable->purchase_id
        ];

        $router = \XF::app()->router('public');

		$purchase->returnUrl = $router->buildLink('canonical:resources/market-place-user/licenses');

        $purchase->cancelUrl = $router->buildLink('canonical:resources/market-place-user/licenses');

        return $purchase;
    }

    public function completePurchase(CallbackState $state)
    {
        if ($state->legacy)
        {
            $purchaseRequest 	= null;
            $purchaseId 		= $state->purchase->purchase_id;
        }
        else
        {
            $purchaseRequest 	= $state->getPurchaseRequest();
            $purchaseId 		= $purchaseRequest->extra_data['purchase_id'];
        }
        
		$paymentResult = $state->paymentResult;

		$purchaseFinder = \XF::finder('XFA\RMMarketplace:Purchase');
		$purchaseFinder->with(['Resource', 'Transaction', 'License']);
		$purchaseFinder->whereId($purchaseId);
		$purchase = $purchaseFinder->fetchOne();
		
		$resource = $purchase->Resource;

        switch ($paymentResult)
        {
            case CallbackState::PAYMENT_RECEIVED:
            case CallbackState::PAYMENT_REINSTATED:
				// Set transaction as completed
				$purchase->Transaction->fastUpdate('status', 'completed');				
            
				// Validate the purchase
                $validator = \XF::service('XFA\RMMarketplace:Purchase\Validate', $purchase);
                $validator->validate();

                $state->logType = 'payment';
                
                if ($paymentResult == CallbackState::PAYMENT_RECEIVED)
                {                
                	$state->logMessage = 'Marketplace renewal payment received, purchase validated.';
				}
				else
				{
                    $state->logMessage = 'Marketplace renewal reversal cancelled, purchase revalidated.';
				}
				
                break;
        }
    }

    public function reversePurchase(CallbackState $state)
    {
        if ($state->legacy)
        {
            $purchaseRequest 	= null;
            $purchaseId 		= $state->purchase->purchase_id;
        }
        else
        {
            $purchaseRequest 	= $state->getPurchaseRequest();
            $purchaseId			= $purchaseRequest->extra_data['purchase_id'];
        }

		$purchaseFinder = \XF::finder('XFA\RMMarketplace:Purchase');
		$purchaseFinder->with(['Resource', 'Transaction', 'License']);
		$purchaseFinder->whereId($purchaseId);
		$purchase = $purchaseFinder->fetchOne();

        /** @var \XFA\RMMarketplace\Service\Purchase\Refund $refunder */
        \XF::service('XFA\RMMarketplace:Purchase\Refund', $purchase)->refund();

        $state->logType = 'cancel';
        $state->logMessage = 'Marketplace renewal payment refunded/reversed, downgraded.';
    }

    public function getPurchasablesByProfileId($profileId)
    {
        return [];
    }
    
    public function sendPaymentReceipt(CallbackState $state)
	{
		return;
	}
}