<?php
/*************************************************************************
 * XFA Core - Xen Factory (c) 2017
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\Service\Purchase;

use XFA\RMMarketplace\Entity\Purchase;

class Notify extends \XF\Service\AbstractService
{
    /**
     * @var Purchase
     */
    protected $purchase;

    public function __construct(\XF\App $app, Purchase $purchase)
    {
        parent::__construct($app);

        $this->purchase = $purchase;
    }

    public function emailBuyer($mailTemplate, $params = [])
    {
        $params['purchase'] = $this->purchase;

        $this->app->mailer()->newMail()
            ->setToUser($this->purchase->Buyer)
            ->setTemplate($mailTemplate, $params)
            ->queue();
    }

    public function pmBuyer($title, $message)
    {
        /** @var \XF\Service\Conversation\Creator $creator */
        $creator = $this->service('XF:Conversation\Creator', $this->purchase->Resource->User);
        $creator->setOptions([
            'open_invite'       => false,
            'conversation_open' => true
        ]);
        $creator->setRecipientsTrusted($this->purchase->Buyer);
        $creator->setContent($title, $message);
        $creator->save();
    }

    public function alertSeller($alertType)
    {
        /** @var \XF\Repository\UserAlert $alertRepo */
        $alertRepo = $this->repository('XF:UserAlert');

        $alertRepo->alert(
            $this->purchase->Resource->User,
            $this->purchase->Buyer->user_id,
            $this->purchase->Buyer->username,
            'xfa_rmmp',
            $this->purchase->resource_id,
            $alertType
        );
    }

    public function communicate($type)
    {
        $seller = $this->purchase->Resource->Seller;

        // If no seller info defined, create an empty seller
        if (!$seller)
        {
            $seller = \XF::app()->em()->create('XFA\RMMarketplace:Seller');
            $seller->hydrateRelation('User', $this->purchase->Resource->User);
        }

        // Get communication data from type
        [$title, $text] = $seller->getCommunicationDataFromType($type);

        // Perform replacements on title
        $title = $this->_replaceResourceName($title);
        $title = $this->_replaceBuyerName($title);

        // Perform replacements on text
        $text = $this->_replaceResourceName($text);
        $text = $this->_replaceResourceLink($text);
        $text = $this->_replaceBuyerName($text);
        $text = $this->_replacePurchasesLink($text);
        $text = $this->_replaceLicensesLink($text);
        $text = $this->_replaceBoardName($text);
        $text = $this->_replaceBoardLink($text);

        // Email buyer depending on seller choice
        if (!$seller->User->hasPermission('general', 'confMarketplaceCommType')
                || $seller->comm_type == 'email'
                || $seller->comm_type == 'both')
        {
            $html = \XF::app()->bbCode()->render(
                $text,
                'emailHtml',
                'email',
                null
            );

            $data = [
                'title'     => $title,
                'textBody'  => $text,
                'htmlBody'  => $html
            ];

            $this->emailBuyer('prepared_email', $data);
        }

        // Start conversation depending on seller choice
        if ($seller->User->hasPermission('general', 'confMarketplaceCommType')
                && ($seller->comm_type == 'pm'
                        || $seller->comm_type == 'both'))
        {
            $this->pmBuyer($title, $text);
        }
    }

    protected function _replaceResourceName($text)
    {
        if (strpos($text, '{resource}') !== false)
        {
            $text = str_replace('{resource}', $this->purchase->Resource->title, $text);
        }

        return $text;
    }

    protected function _replaceResourceLink($text)
    {
        if (strpos($text, '{resource_link}') !== false)
        {
            $url = \XF::app()->router('public')->buildLink('canonical:resources', $this->purchase->Resource);
            $rep = '[URL=' . $url . ']' . $this->purchase->Resource->title . '[/URL]';

            $text = str_replace('{resource_link}', $rep, $text);
        }

        return $text;
    }

    protected function _replaceBuyerName($text)
    {
        if (strpos($text, '{username}') !== false)
        {
            $text = str_replace('{username}', $this->purchase->Buyer->username, $text);
        }

        return $text;
    }

    protected function _replacePurchasesLink($text)
    {
        if (strpos($text, '{purchases_link}') !== false)
        {
            $url = \XF::app()->router('public')->buildLink('canonical:resources/market-place-user/purchases', $this->purchase->Buyer);
            $rep = '[URL=' . $url . ']' . \XF::phrase('xfa_rmmp_your_purchases')->render() . '[/URL]';

            $text = str_replace('{purchases_link}', $rep, $text);
        }

        return $text;
    }

    protected function _replaceLicensesLink($text)
    {
        if (strpos($text, '{licenses_link}') !== false)
        {
            $url = \XF::app()->router('public')->buildLink('canonical:resources/market-place-user/licenses', $this->purchase->Buyer);
            $rep = '[URL=' . $url . ']' . \XF::phrase('xfa_rmmp_your_licenses')->render() . '[/URL]';

            $text = str_replace('{licenses_link}', $rep, $text);
        }

        return $text;
    }

    protected function _replaceBoardName($text)
    {
        if (strpos($text, '{board}') !== false)
        {
            $text = str_replace('{board}', \XF::options()->boardTitle, $text);
        }

        return $text;
    }

    protected function _replaceBoardLink($text)
    {
        if (strpos($text, '{board_link}') !== false)
        {
            $url = \XF::app()->router('public')->buildLink('canonical:index');
            $rep = '[URL=' . $url . ']' . \XF::options()->boardTitle . '[/URL]';

            $text = str_replace('{board_link}', $rep, $text);
        }

        return $text;
    }
}