<?php

namespace XFA\RMMarketplace\Service\ResourceItem;

use XFRM\Entity\ResourceItem;

class Image extends \XF\Service\AbstractService
{
    /**
     * @var \XFRM\Entity\ResourceItem
     */
    protected $resource;

    protected $logIp = true;

    protected $fileName;

    protected $width;

    protected $height;

    protected $type;

    protected $error = null;

    protected $allowedTypes = [IMAGETYPE_GIF, IMAGETYPE_JPEG, IMAGETYPE_PNG];


    public function __construct(\XF\App $app, ResourceItem $resource)
    {
        parent::__construct($app);
        $this->resource = $resource;
    }

    public function getResource()
    {
        return $this->resource;
    }

    public function logIp($logIp)
    {
        $this->logIp = $logIp;
    }

    public function getError()
    {
        return $this->error;
    }

    public function setImage($fileName)
    {
        if (!$this->validateImage($fileName, $error))
        {
            $this->error = $error;
            $this->fileName = null;
            return false;
        }

        $this->fileName = $fileName;
        return true;
    }

    public function setImageFromUpload(\XF\Http\Upload $upload)
    {
        $upload->requireImage();

        if (!$upload->isValid($errors))
        {
            $this->error = reset($errors);
            return false;
        }

        return $this->setImage($upload->getTempFile());
    }

    public function validateImage($fileName, &$error = null)
    {
        $error = null;

        if (!file_exists($fileName))
        {
            throw new \InvalidArgumentException("Invalid file '$fileName' passed to image service");
        }
        if (!is_readable($fileName))
        {
            throw new \InvalidArgumentException("'$fileName' passed to image service is not readable");
        }

        $imageInfo = filesize($fileName) ? getimagesize($fileName) : false;
        if (!$imageInfo)
        {
            $error = \XF::phrase('provided_file_is_not_valid_image');
            return false;
        }

        $type = $imageInfo[2];
        if (!in_array($type, $this->allowedTypes))
        {
            $error = \XF::phrase('provided_file_is_not_valid_image');
            return false;
        }

        $width = $imageInfo[0];
        $height = $imageInfo[1];

        if (!$this->app->imageManager()->canResize($width, $height))
        {
            $error = \XF::phrase('uploaded_image_is_too_big');
            return false;
        }

        $this->width    = $width;
        $this->height   = $height;
        $this->type     = $type;

        return true;
    }

    public function updateImage()
    {
        if (!$this->fileName)
        {
            throw new \LogicException("No source file for image set");
        }

        $imageManager   = $this->app->imageManager();
        $outputFile     = null;

        if ($this->width > 250 || $this->height > 350)
        {
            $image = $imageManager->imageFromFile($this->fileName);
            if (!$image)
            {
                return false;
            }

            $image->resizeAndCrop(250, 350);

            $newTempFile = \XF\Util\File::getTempFile();
            if ($newTempFile && $image->save($newTempFile))
            {
                $outputFile = $newTempFile;
            }
        }
        else
        {
            $outputFile = $this->fileName;
        }

        if (!$outputFile)
        {
            throw new \RuntimeException("Failed to save image to temporary file; check internal_data/data permissions");
        }

        $dataFile = $this->resource->getAbstractedImagePath();
        \XF\Util\File::copyFileToAbstractedPath($outputFile, $dataFile);

        $this->resource->xfa_rmmp_img_date = \XF::$time;
        $this->resource->save();

        if ($this->logIp)
        {
            $ip = ($this->logIp === true ? $this->app->request()->getIp() : $this->logIp);
            $this->writeIpLog('update', $ip);
        }

        return true;
    }

    public function deleteImage()
    {
        $this->deleteImageFiles();

        $this->resource->xfa_rmmp_img_date = 0;
        $this->resource->save();

        if ($this->logIp)
        {
            $ip = ($this->logIp === true ? $this->app->request()->getIp() : $this->logIp);
            $this->writeIpLog('delete', $ip);
        }

        return true;
    }

    public function deleteImageForResourceDelete()
    {
        $this->deleteImageFiles();

        return true;
    }

    protected function deleteImageFiles()
    {
        if ($this->resource->icon_date)
        {
            \XF\Util\File::deleteFromAbstractedPath($this->resource->getAbstractedImagePath());
        }
    }

    protected function writeIpLog($action, $ip)
    {
        $resource = $this->resource;

        /** @var \XF\Repository\Ip $ipRepo */
        $ipRepo = $this->repository('XF:Ip');
        $ipRepo->logIp(\XF::visitor()->user_id, $ip, 'resource', $resource->resource_id, 'image_' . $action);
    }
}