<?php
/*************************************************************************
 * XenForo RM Donations - Xen Factory (c) 2015-2019
 * All Rights Reserved.
 * Created by Clement Letonnelier aka. MtoR
 *************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at http://xen-factory.com/pages/license-agreement/.
 *************************************************************************/

namespace XFA\RMMarketplace\XFRM\Pub\Controller;

use XF\Mvc\ParameterBag;

class Category extends XFCP_Category
{
    public function actionAdd(ParameterBag $params)
    {
        $reply = parent::actionAdd($params);

        if (!$this->isPost())
        {
            // If user can send child resources get user resources
            $category = $reply->getParam('category');

            if ($category->canSellChildResource())
            {
                $reply->setParam(
                    'userResources',
                    $this->repository('XFRM:ResourceItem')->findPaidResourcesByUserOptions(\XF::visitor()->user_id)
                );
            }
        }

        return $reply;
    }

    protected function setupResourceCreate(\XFRM\Entity\Category $category)
    {
        $creator = parent::setupResourceCreate($category);

        if ($this->filter('resource_payment', 'str') == 'resource_for_sale' && $category->canAskForPayment() && !$category->hasReachedMaxNumberOfSales())
        {
            /* If resource sale is authorized, get the selected product type */
            $productType = $this->filter('xfa_rmmp_type', 'str');

            /* Check product type is authorized for the category */
            switch ($productType) 
            {
                case 'physical':
                    if ($category->xfa_rmmp_types['physical'] == 0)
                    {
                        $creator->getResource()->error(\XF::phrase('xfa_rmmp_product_type_not_allowed_in_category'));
                        return $creator;
                    }
                    break;
                case 'digital':
                    if ($category->xfa_rmmp_types['digital']['active'] == 0)
                    {
                        $creator->getResource()->error(\XF::phrase('xfa_rmmp_product_type_not_allowed_in_category'));
                        return $creator;
                    }
                    break;
                case 'service':
                    if ($category->xfa_rmmp_types['service'] == 0)
                    {
                        $creator->getResource()->error(\XF::phrase('xfa_rmmp_product_type_not_allowed_in_category'));
                        return $creator;
                    }
                    break;
            }

            /* Cleanup */
            $saleData = $this->filter([
                'xfa_rmmp_parent_id'    => 'uint',
                'xfa_rmmp_type'         => 'str',
                'xfa_rmmp_max_qty'      => 'uint',
                'xfa_rmmp_amount'       => 'str',
                'xfa_rmmp_accept_other' => 'bool',
                'xfa_rmmp_currency'     => 'str'
            ]);

            $saleData['xfa_rmmp_currency'] = strtoupper($saleData['xfa_rmmp_currency']);

            /* Check minimum price */
            if ((\XF::options()->xfa_rmmp_minimumPrice && $saleData['xfa_rmmp_amount'] < \XF::options()->xfa_rmmp_minimumPrice)
                    || $saleData['xfa_rmmp_amount'] == 0)
            {
                $creator->getResource()->error(\XF::phrase('xfa_rmmp_price_below_minimum_required_value'));
                return $creator;
            }

            /* Check if price is input with decimal as coma instead of point */
            if (strpos($saleData['xfa_rmmp_amount'], ',') !== false)
            {
                $saleData['xfa_rmmp_amount'] = str_replace(',', '.', $saleData['xfa_rmmp_amount']);
            }

            /* Handle terms and conditions */
            $saleData['xfa_rmmp_cgv'] = $this->plugin('XF:Editor')->fromInput('xfa_rmmp_cgv');

            /* Get email address only if not set globally */
            if ($category->xfa_rmmp_conf == 1)
            {
                $saleData['xfa_rmmp_paypal'] = $this->filter('xfa_rmmp_paypal', 'str');
            }

            /* Add marketplace date to resource */
            $creator->getResource()->bulkSet($saleData);
        }
        else
        {
            $creator->getResource()->xfa_rmmp_type = 'none';
        }

        return $creator;
    }

    protected function finalizeResourceCreate(\XFRM\Service\ResourceItem\Create $creator)
    {
        parent::finalizeResourceCreate($creator);

        $resource = $creator->getResource();
        $category = $creator->getCategory();

        /* If resource save completed, create product depending on its type, expect for service where nothing needs to be done */
        switch ($resource->xfa_rmmp_type)
        {
            /* Physical Product */
            case 'physical':
                $inputData = $this->filter([
                    'quantity'          => 'uint',
                    'sell_on_stockout'  => 'uint'
                ]);

                $inputData['resource_id'] = $resource->resource_id;

                // Handle shipping costs
                $filteredShippingCosts = $this->filter('shipping', 'array');

                $shippingCosts = [];
                foreach ($filteredShippingCosts AS $shippingUnfiltered)
                {
                    $shipping = $this->filterArray($shippingUnfiltered, ['location' => 'str', 'cost' => 'float']);

                    if (!empty($shipping['location']))
                    {
                        $shippingCosts[] = $shipping;
                    }
                }

                $inputData['shipping'] = $shippingCosts;

                $physicalProduct = $this->em()->create('XFA\RMMarketplace:PhysicalProduct');
                $physicalProduct->bulkSet($inputData);
                $physicalProduct->save();

                break;

            /* Digital product */
            case 'digital':
                $inputData = [];

                $inputData['resource_id'] = $resource->resource_id;

                if ($category->xfa_rmmp_types['digital']['renewal'])
                {
                    $inputData['duration']              = $this->filter('duration', 'uint');
                    $inputData['duration_type']         = $this->filter('duration_type', 'str');
                    $inputData['delay_renewal']         = $this->filter('delay_renewal', 'uint');
                    $inputData['allow_expired_renewal'] = $this->filter('allow_expired_renewal', 'uint');
                    $inputData['renewal_price']         = $this->filter('renewal_price', 'float');
                }
                else
                {
                    $inputData['duration']              = 0;
                    $inputData['delay_renewal']         = 0;
                    $inputData['allow_expired_renewal'] = 1;
                    $inputData['renewal_price']         = 0;
                }

                if ($category->xfa_rmmp_types['digital']['restriction'])
                {
                    $inputData['has_restriction']   = $this->filter('has_restriction', 'bool');
                }
                else
                {
                    $inputData['has_restriction']   = 0;
                }

                if ($category->xfa_rmmp_types['digital']['url_required'])
                {
                    $inputData['require_url']       = $this->filter('require_url', 'bool');
                }
                else
                {
                    $inputData['require_url']       = 0;
                }

                if ($category->canActivateAutoSerialKeyGen())
                {
                    $inputData['lic_key_format'] = $this->filter('lic_key_format', 'str');
                }
                else
                {
                    $inputData['lic_key_format'] = "";
                }

                $digitalProduct = $this->em()->create('XFA\RMMarketplace:DigitalProduct');
                $digitalProduct->bulkSet($inputData);
                $digitalProduct->save();

                break;
        }

        if ($resource->canEditResourceImage())
        {
            try
            {
                /** @var \XFA\RMMarketplace\ControllerPlugin\ResourceImage $plugin */
                $plugin = $this->plugin('XFA\RMMarketplace:ResourceImage');
                $plugin->actionUpload($resource);
            }
            catch (\XF\Mvc\Reply\Exception $e)
            {
            }
        }
    }
}